@@ -1,9 +1,9 @@
 ;; -*- emacs-lisp -*-
+;;
 ;; mew-caesar.el --- Caesar encrypt/decrypt assistant package for Mew.
 ;;
-;;                         "Hideyuki SHIRAI" <Shirai@rdmg.mgcs.mei.co.jp>
-;;                                            Created: <02/07/1998>
-;;                                Revised: Time-stamp: <04/02/1999 18:27 shirai>
+;;   Author: Hideyuki SHIRAI <shirai@mew.org>
+;;   Created: <02/07/1998>
 ;;
 ;; To use mew-caesar.el, install (tm|SEMI) package or "nkf"
 ;;  , and put the following codes in your .emacs.
@@ -80,8 +80,8 @@
 	 ("text/x-rot13.*" "\\.rot$" nil mew-caesar-prog-xrot mew-icon-text))
        mew-mime-content-type))
 
-(defun mew-caesar-mime-text/x-rot (begin end &optional params execute)
-  (if (> end begin)
+(defun mew-caesar-mime-text/x-rot (cache beg end &optional params execute)
+  (if (> end beg)
       (save-excursion
 	(set-buffer (mew-buffer-message))
 	(let ((buffer-read-only nil))
@@ -100,35 +100,33 @@
 		  (substitute-command-keys
 		   "\\<mew-summary-mode-map>\\[mew-caesar-summary-insert-xrot]."))
 	  (insert "\n\n-------------------- Original \"X-ROT13\" follows --------------------\n")
-	  (insert-buffer-substring (mew-current-get 'cache) begin end)
+	  (insert-buffer-substring cache beg end)
 	  ))))
 
 (defun mew-caesar-summary-insert-xrot ()
   (interactive)
-  (let* ((ofld-msg (mew-current-get 'message))
-	 (msg (mew-summary-message-number))
-	 (nums (mew-syntax-nums))
-	 (buf (buffer-name)))
-    (if (or msg (not nums))
-	(let ((mew-analysis nil))
-	  (mew-summary-display 'force))
-      (unwind-protect
-	  (progn
-	    (mew-summary-toggle-disp-msg 'on)
-	    (mew-window-configure buf 'message)
-	    (set-buffer (mew-buffer-message))
-	    (let* ((buffer-read-only nil)
-		   (syntax (mew-cache-decode-syntax (mew-cache-hit ofld-msg)))
-		   (stx (mew-syntax-get-entry syntax nums))
-		   (begin (mew-syntax-get-begin stx))
-		   (end (mew-syntax-get-end stx)))
-	      (erase-buffer)
-	      (insert-buffer-substring (mew-current-get 'cache) begin end)
-	      (mew-caesar-whole-buffer)
-	      (mew-message-set-end-of)
-	      (goto-char (point-min))))
-	(mew-pop-to-buffer buf)))
-    ))
+  (mew-summary-part
+   (let* ((fld (mew-current-get-fld (mew-frame-id)))
+	  (msg (mew-current-get-msg (mew-frame-id)))
+	  (part (mew-syntax-nums))
+	  (cache (mew-cache-hit fld msg 'must-hit))
+	  (syntax (mew-cache-decode-syntax cache))
+	  (stx (mew-syntax-get-entry syntax part))
+	  (beg (mew-syntax-get-begin stx))
+	  (end (mew-syntax-get-end stx))
+	  (buf (current-buffer)))
+     (unwind-protect
+	 (progn
+	   (mew-summary-toggle-disp-msg 'on)
+	   (mew-window-configure 'message)
+	   (set-buffer (mew-buffer-message))
+	   (mew-elet
+	    (mew-summary-display-preamble)
+	    (insert-buffer-substring cache beg end)
+	    (mew-caesar-whole-buffer)
+	    ;; (goto-char (point-min))
+	    (mew-summary-display-postscript)))
+       (mew-pop-to-buffer buf)))))
 
 (defun mew-caesar-attach-find-new-xrot ()
   "Open a new Caesar encrypt file into a buffer on \".\" in attachments."
@@ -137,13 +135,13 @@
       (message "Can't find a new file here.")
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
-	   (mimedir (mew-expand-folder (mew-draft-to-mime (buffer-name))))
+	   (attachdir (mew-attachdir))
 	   file filepath)
-      ;; mimedir / {subdir/} dir
+      ;; attachdir / {subdir/} dir
       (if (not (equal subdir ""))
-	  (setq mimedir (expand-file-name subdir mimedir)))
-      ;; mimedir / file
-      (setq filepath (mew-random-filename mimedir mew-caesar-rot13-suffix))
+	  (setq attachdir (expand-file-name subdir attachdir)))
+      ;; attachdir / file
+      (setq filepath (mew-random-filename attachdir 1 nil mew-caesar-rot13-suffix))
       (if (null filepath)
 	  (message "Could not make a text file, sorry.")
 	(setq file (file-name-nondirectory filepath))
@@ -206,17 +204,49 @@
 	(tm:caesar-region)))
      ((and (eq mew-caesar-function 'ext)
 	   mew-caesar-ext-prog mew-caesar-ext-prog-arg)
-      (let ((input-coding-system mew-cs-autoconv)
-	    (output-coding-system mew-cs-7bit)
-	    (coding-system-for-read mew-cs-autoconv)
-	    (coding-system-for-write mew-cs-7bit))
-	(apply 'call-process-region min max
-	       mew-caesar-ext-prog 
-	       t t nil
-	       mew-caesar-ext-prog-arg)))
+      (save-excursion
+	(mew-piolet
+	 mew-cs-autoconv mew-cs-m17n
+	 (apply (function call-process-region)
+		min max
+		mew-caesar-ext-prog 
+		t t nil
+		mew-caesar-ext-prog-arg))))
      (t
       (message "mew-caesar: program is not found.")))
     ))
 
 (provide 'mew-caesar)
-;;
+
+;;; Copyright Notice:
+
+;; Copyright (C) 1998-2000 Hideyuki SHIRAI <shirai@mew.org>
+;; Copyright (C) 1994-2000 Mew developing team.
+;; All rights reserved.
+
+;; Redistribution and use in source and binary forms, with or without
+;; modification, are permitted provided that the following conditions
+;; are met:
+;; 
+;; 1. Redistributions of source code must retain the above copyright
+;;    notice, this list of conditions and the following disclaimer.
+;; 2. Redistributions in binary form must reproduce the above copyright
+;;    notice, this list of conditions and the following disclaimer in the
+;;    documentation and/or other materials provided with the distribution.
+;; 3. Neither the name of the team nor the names of its contributors
+;;    may be used to endorse or promote products derived from this software
+;;    without specific prior written permission.
+;; 
+;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
+;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+;; PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE TEAM OR CONTRIBUTORS BE
+;; LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+;; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+;; BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+;; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+;; IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+;;; mew-caesar.el ends here
