@@ -2,63 +2,12 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 30, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-attach-version "mew-attach.el version 0.18")
-
 (require 'mew)
 
-(defvar mew-draft-attach-map nil
-  "A region map for attachments in Draft mode")
-
-(if mew-draft-attach-map
-    ()
-  (setq mew-draft-attach-map (make-keymap))
-  (define-key mew-draft-attach-map "\C-m" 'mew-attach-newline)
-  (define-key mew-draft-attach-map "\C-f" 'mew-attach-forward)
-  (define-key mew-draft-attach-map "\C-b" 'mew-attach-backforward)
-  (define-key mew-draft-attach-map "\C-n" 'mew-attach-next)
-  (define-key mew-draft-attach-map "\C-p" 'mew-attach-previous)
-  (define-key mew-draft-attach-map "a"    'mew-attach-audio)
-  (define-key mew-draft-attach-map "c"    'mew-attach-copy)
-  (define-key mew-draft-attach-map "C"    'mew-attach-charset)
-  (define-key mew-draft-attach-map "d"    'mew-attach-delete)
-  (define-key mew-draft-attach-map "y"    'mew-attach-link-message)
-  (define-key mew-draft-attach-map "D"    'mew-attach-description)
-  (define-key mew-draft-attach-map "P"    'mew-attach-disposition)
-  (define-key mew-draft-attach-map "e"    'mew-attach-external-body)
-  (define-key mew-draft-attach-map "f"    'mew-attach-find-file)
-  (define-key mew-draft-attach-map "F"    'mew-attach-find-new-file)
-  (define-key mew-draft-attach-map "l"    'mew-attach-link)
-  (define-key mew-draft-attach-map "m"    'mew-attach-multipart)
-  (define-key mew-draft-attach-map "T"    'mew-attach-type)
-  (define-key mew-draft-attach-map "G"    'mew-attach-gzip64)
-  (define-key mew-draft-attach-map "B"    'mew-attach-base64)
-  (define-key mew-draft-attach-map "Q"    'mew-attach-quoted-printable)
-  (define-key mew-draft-attach-map "S"    'mew-attach-pgp-sign)
-  (define-key mew-draft-attach-map "E"    'mew-attach-pgp-enc)
-  (define-key mew-draft-attach-map "p"    'mew-attach-pgp-public-key)
-  (define-key mew-draft-attach-map "U"    'mew-attach-undo)
-  (if mew-use-overlay-keymap
-      (mew-draft-share-keymap 'mew-draft-attach-map))
-  )
-
-(if mew-xemacs-p
-    ()
-  (if mew-use-overlay-keymap
-      (easy-menu-define
-       mew-draft-attach-menu
-       mew-draft-attach-map
-       "Menu used in Draft mode."
-       mew-draft-mode-menu-spec)))
-
-(defconst mew-draft-attach-boundary-begin
-  "------------------------------ attachments ------------------------------")
-(defconst mew-draft-attach-boundary-end
-  "--------0-1-2-3-4-5-6-7-8-9----------------------------------------------")
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; goodies
@@ -69,20 +18,19 @@
 To compose a multipart message, you should execute this command first."
   (interactive)
   (if (not (mew-header-p))
-      (message "Attachments region can be prepared before makeing a message only")
+      (message "Attachments region can be prepared before making a message only")
     (if (mew-attach-p)
-	(message "Attachemnts region have been already prepared")
+	(message "Attachments region have been already prepared")
       (let* ((attachdir (mew-attachdir))
 	     (cp (expand-file-name mew-draft-coverpage attachdir))
 	     (dir (file-name-nondirectory attachdir)))
 	(if (mew-attach-p)
 	    (message "Attachments already exist.")
 	  (mew-attach-set)
-	  (if (not (file-exists-p attachdir)) (mew-make-directory attachdir))
-	  (if (not (file-exists-p cp)) 
-	      (write-region 1 1 cp 'append 'no-msg));; touch
-	  (if (not mew-encode-syntax)
-	      (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
+	  (unless (file-exists-p attachdir) (mew-make-directory attachdir))
+	  (unless (file-exists-p cp) (write-region "" nil cp 'append 'no-msg))
+	  (unless mew-encode-syntax
+	    (setq mew-encode-syntax (mew-encode-syntax-initial dir)))
 	  (mew-encode-syntax-print mew-encode-syntax)
 	  (mew-attach-goto-number 'here '(2))
 	  (if (and (not no-scroll)
@@ -110,10 +58,10 @@ attachments region.
   (let (ret max)
     (save-excursion
       (beginning-of-line)
-      (setq ret (count-lines (1+ (mew-attach-begin)) (point)))
+      (setq ret (mew-count-lines (1+ (mew-attach-begin)) (point)))
       (goto-char (point-max))
       (beginning-of-line)
-      (setq max (count-lines (1+ (mew-attach-begin)) (point)))
+      (setq max (mew-count-lines (1+ (mew-attach-begin)) (point)))
       (if (or (= ret max) (= ret (1- max))) -1 ret))))
 
 
@@ -122,7 +70,7 @@ attachments region.
     (save-excursion
       (goto-char (point-max))
       (beginning-of-line)
-      (equal (count-lines here (point)) 2))))
+      (= (mew-count-lines here (point)) 2))))
 
 (defun mew-attach-move-onto-file ()
   (end-of-line)
@@ -141,65 +89,63 @@ attachments region.
 (defun mew-attach-dot-p ()
   (save-excursion
     (mew-attach-move-onto-file)
-    (looking-at "\\.$")))
+    (looking-at "\\. *$")))
 
 ;; cursor only
 (defun mew-attach-line0-1 ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(or (= line 0) (= line -1)))))
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (or (= line 0) (= line -1)))))
 
 ;; cursor only
 (defun mew-attach-line01-1 ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(or (= line 0) (= line 1) (= line -1)))))
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (or (= line 0) (= line 1) (= line -1)))))
 
 ;; insert commands
 (defun mew-attach-not-line012-1 ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(not (or (= line 0) (= line 1) (= line 2) (= line -1))))))
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (not (or (= line 0) (= line 1) (= line 2) (= line -1))))))
 
 ;; find commands
 (defun mew-attach-not-line012-1-dot ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(not (or (mew-attach-dot-p)
-		 (= line 0) (= line 1) (= line 2) (= line -1))))))
-
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (not (or (mew-attach-dot-p)
+	       (= line 0) (= line 1) (= line 2) (= line -1))))))
 
 ;; delete commands
- 
 (defun mew-attach-not-line02-1-dot ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(not (or (mew-attach-dot-p) (= line 0) (= line 2) (= line -1))))))
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (not (or (mew-attach-dot-p) (= line 0) (= line 2) (= line -1))))))
 
 
 ;; modify commands
 (defun mew-attach-not-line0-1-dot ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(not (or (mew-attach-dot-p) (= line 0) (= line -1))))))
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (not (or (mew-attach-dot-p) (= line 0) (= line -1))))))
 
-;; corsor
+;; cursor
 (defun mew-attach-line1-dot ()
-  (if (mew-attach-p)
-      (let ((line (mew-attach-line)))
-	(or (mew-attach-dot-p) (= line 1)))))
+  (when (mew-attach-p)
+    (let ((line (mew-attach-line)))
+      (or (mew-attach-dot-p) (= line 1)))))
 
 (defun mew-attach-goto-number (direction nums)
   (let (numreg)
     (setq nums (nreverse nums))
     (cond
-     ((equal direction 'next)
+     ((eq direction 'next)
       (setq nums (cons (1+ (car nums)) (cdr nums))))
-     ((equal direction 'prev)
+     ((eq direction 'prev)
       (setq nums (cons (1- (car nums)) (cdr nums))))
-     ((equal direction 'up)
+     ((eq direction 'up)
       (setq nums (cdr nums)))
-     ((equal direction 'down)
+     ((eq direction 'down)
       (setq nums (cons 1 nums)))
      (t ()))
     (if (null nums)
@@ -229,10 +175,9 @@ attachments region.
 (defun mew-attach-newline ()
   "Insert RET before the attachments region."
   (interactive)
-  (if (equal (point) (1+ (mew-attach-begin)))
-      (progn
-	(insert "\n")
-	(mew-draft-attach-keymap))))
+  (when (= (point) (1+ (mew-attach-begin)))
+    (insert "\n")
+    (mew-draft-attach-keymap)))
 
 (defun mew-attach-forward ()
   "Go to the first subdirectory in attachments."
@@ -292,31 +237,29 @@ attachments region.
   (interactive)
   (if (not (mew-attach-not-line02-1-dot))
       (message "Can't delete here.")
-    (if (equal (mew-attach-line) 1)
-	(if (y-or-n-p "Delete entire attachments? ")
-	    (progn
-	      (mew-delete-directory-recursively (mew-attachdir))
-	      (setq mew-encode-syntax nil)
-	      (mew-attach-clear)
-	      (if mew-icon-p
-		  (set-specifier default-toolbar
-				 (cons (current-buffer) mew-draft-toolbar)))
-	      (message "attachments were deleted"))
-	  (message "attachments were not deleted"))
+    (if (= (mew-attach-line) 1)
+	(if (not (y-or-n-p "Delete entire attachments? "))
+	    (message "attachments were not deleted")
+	  (mew-delete-directory-recursively (mew-attachdir))
+	  (setq mew-encode-syntax nil)
+	  (mew-attach-clear)
+	  (if (and mew-xemacs-p mew-icon-p)
+	      (set-specifier default-toolbar
+			     (cons (current-buffer) mew-draft-toolbar)))
+	  (message "attachments were deleted"))
       (let* ((nums (mew-syntax-nums))
 	     (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	     (attachdir (mew-attachdir))
 	     (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	     (name (mew-syntax-get-file syntax))
-	     (ename (if (equal subdir "") name (concat subdir name)))
+	     (ename (if (string= subdir "") name (concat subdir name)))
 	     (fullname (expand-file-name ename attachdir))
 	     (dirp (string-match (concat mew-path-separator "$") name))
 	     (msg (if dirp
 		      "Delete %s with contained files? "
 		    "Delete %s? ")))
 	;; attachdir / {subdir/} name
-	(if (not (y-or-n-p (format msg ename)))
-	    ()
+	(when (y-or-n-p (format msg ename))
 	  (message "Deleting %s ... " ename)
           (if (file-exists-p fullname)
               (if dirp
@@ -342,10 +285,10 @@ attachments region.
 	   (attachdir (mew-attachdir))
 	   dir dirpath)
       ;; attachdir / {subdir/} dir
-      (if (not (equal subdir "")) 
-	  (setq attachdir (expand-file-name subdir attachdir)))
+      (unless (string= subdir "")
+	(setq attachdir (expand-file-name subdir attachdir)))
       ;; attachdir / dir
-      (setq dirpath (mew-random-filename attachdir))
+      (setq dirpath (mew-random-filename attachdir 1 nil))
       (if (file-exists-p dirpath)
 	  (message "Could not make a directory, sorry")
 	(setq dir (file-name-nondirectory dirpath))
@@ -370,11 +313,11 @@ attachments region.
       (while (or (null frompath)
 		 (file-directory-p (file-chase-links frompath)))
 	(setq frompath (mew-draft-input-file-name
-			(concat (capitalize action) " from : ") default)))
+			(concat (mew-capitalize action) " from : ") default)))
       (while (or (null efile)
 		 (file-exists-p (expand-file-name efile attachdir)))
 	(setq tofile (mew-input-string 
-		      (concat (capitalize action) " to %s(%s): ")
+		      (concat (mew-capitalize action) " to %s(%s): ")
 		      subdir
 		      (file-name-nondirectory frompath)))
 	(setq efile (concat subdir tofile)))
@@ -406,10 +349,10 @@ To copy a remote file, use the \"/[user@]hostname:/filepath\" syntax."
   "Link the message displayed in Message mode with a symbolic link
 on '.' in attachments."
   (interactive)
-  (let ((fld-msg (mew-current-get 'message))
+  (let ((fld (mew-current-get-fld (mew-frame-id)))
+	(msg (mew-current-get-msg (mew-frame-id)))
 	default)
-    (if fld-msg
-	(setq default (mew-expand-folder-get-msg (car fld-msg) (cdr fld-msg))))
+    (if fld (setq default (mew-expand-folder fld msg)))
     (if mew-use-symbolic-link-for-forwarding
 	(mew-attach-duplicate (function mew-symbolic-link) "link" nil nil nil default)
       (mew-attach-duplicate (function copy-file) "copy" nil nil nil default))))
@@ -424,10 +367,10 @@ on '.' in attachments."
 	   (attachdir (mew-attachdir))
 	   file filepath)
       ;; attachdir / {subdir/} dir
-      (if (not (equal subdir "")) 
-	  (setq attachdir (expand-file-name subdir attachdir)))
+      (unless (string= subdir "")
+	(setq attachdir (expand-file-name subdir attachdir)))
       ;; attachdir / file
-      (setq filepath (mew-random-filename attachdir mew-text-suffix))
+      (setq filepath (mew-random-filename attachdir 1 nil mew-text-suffix))
       (if (null filepath)
 	  (message "Could not make a text file, sorry.")
 	(setq file (file-name-nondirectory filepath))
@@ -452,24 +395,26 @@ on '.' in attachments."
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
+	   (buf (generate-new-buffer mew-buffer-prefix))
 	   file filepath recorded)
       ;; attachdir / {subdir/} dir
-      (if (not (equal subdir "")) 
-	  (setq attachdir (expand-file-name subdir attachdir)))
+      (unless (string= subdir "")
+	(setq attachdir (expand-file-name subdir attachdir)))
       ;; attachdir / file
-      (setq filepath (mew-random-filename attachdir mew-audio-suffix))
+      (setq filepath (mew-random-filename attachdir 1 nil mew-audio-suffix))
       (if (null filepath)
 	  (message "Could not make a file for audio, sorry.")
 	(setq file (file-name-nondirectory filepath))
 	(save-excursion
-	  (mew-set-buffer-tmp)
+	  (set-buffer buf)
+	  (mew-erase-buffer)
 	  (condition-case nil
 	      (progn
 		(if (not (y-or-n-p "Are you ready? "))
 		    (message "Nothing is recoded.")
 		  (message
 		   (substitute-command-keys
-		    "Type '\\[keyboard-quit]' to finish recording..."))
+		    "Type '\\[keyboard-quit]' to finish recording ... "))
 		  (mew-plet
 		   (apply (function call-process)
 			  (nth 0 mew-prog-audio2)
@@ -482,6 +427,7 @@ on '.' in attachments."
 	     (setq recorded t)
 	     (mew-flet
 	      (write-region (point-min) (point-max) filepath nil 'no-msg)))))
+	(mew-remove-buffer buf)
 	(if recorded
 	    (setq mew-encode-syntax ;; buffer local
 		  (mew-syntax-insert-entry
@@ -527,15 +473,24 @@ on '.' in attachments."
 				 mew-mime-content-type-list)))
       (setq ctl (list ct))
       (mew-syntax-set-ct syntax ctl)
-      (setq cte (mew-attr-get-cte (mew-attr-by-ct ct)))
+      (setq cte (mew-ctdb-cte (mew-ctdb-by-ct ct)))
       (mew-syntax-set-cte syntax cte)
+      (mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file))
       (mew-encode-syntax-print mew-encode-syntax))))
 
 (defun mew-attach-charset ()
-  "Specify charset for a Text/* object in attachments."
+  "Specify the charset parameter for a Text/* object in attachments."
   (interactive)
+  (mew-attach-charset-base "charset"))
+
+(defun mew-attach-icharset ()
+  "Specify a input coding-system for a text file in attachments."
+  (interactive)
+  (mew-attach-charset-base "icharset"))
+
+(defun mew-attach-charset-base (param)
   (if (not mew-mule-p)
-      (message "This commands cannot be used on this Emacs 19")
+      (message "This commands cannot be used on this Emacs")
     (if (not (mew-attach-not-line0-1-dot))
 	(message "Can't specify character set here")
       (let* ((nums (mew-syntax-nums))
@@ -543,49 +498,55 @@ on '.' in attachments."
 	     (file (mew-syntax-get-file syntax))
 	     (ctl (mew-syntax-get-ct syntax))
 	     (ct (mew-syntax-get-value ctl 'cap))
-	     (char (mew-syntax-get-param ctl "charset"))
-	     charset
-	     (case-fold-search t))
-	(if (not (string-match "text/" ct))
+	     (params (mew-syntax-get-params ctl))
+	     (char (mew-syntax-get-param ctl param))
+	     charset)
+	(if (not (mew-ct-textp ct))
 	    (message "Can't specify character set to %s" ct)
-	  (if (null char) (setq char "guess"))
-	  (setq charset (mew-input-type "Charset for %s (%s): "
-					file char mew-charset-list))
-	  (if (equal charset "guess")
-	      (setq ctl (list ct))
-	    (setq ctl (list ct (list "charset" charset))))
+	  (unless char (setq char "guess"))
+	  (setq charset (mew-input-type
+			 (if (string= param "charset")
+			     "Output message charset for %s (%s): "
+			   "Input file charset for %s (%s): ")
+			 file char
+			 (cons "guess" mew-charset-list)))
+	  (setq params (mew-delete param params))
+	  (if (string= charset "guess")
+	      (setq ctl (cons ct params))
+	    (setq ctl (cons ct (cons (list param charset) params))))
 	  (mew-syntax-set-ct syntax ctl)
 	  (mew-encode-syntax-print mew-encode-syntax))))))
 
-(defun mew-attach-description (&optional cd)
+(defun mew-attach-description ()
   "Input a description(Content-Description:) in attachments."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
       (message "Can't describe here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
-	   (ocd (or (mew-syntax-get-cd syntax) (mew-syntax-get-file syntax))))
-      (if (null cd)
-	  (setq cd (read-string "Description: " ocd)))
-      (if (equal cd "")
+	   (ocd (or (mew-syntax-get-cd syntax) (mew-syntax-get-file syntax)))
+	   (cd (read-string "Description: " ocd)))
+      (if (string= cd "")
 	  (mew-syntax-set-cd syntax nil)
 	(mew-syntax-set-cd syntax cd))
       (mew-encode-syntax-print mew-encode-syntax))))
 
-(defun mew-attach-disposition (&optional cdp)
+(defun mew-attach-disposition ()
   "Input Content-Disposition: in attachments."
   (interactive)
-  (if (not (mew-attach-not-line012-1-dot))
+  (if (not (mew-attach-not-line0-1-dot))
       (message "Can't set disposition here")
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
-	   (ocdp (or (mew-syntax-get-param (mew-syntax-get-cdp syntax)
-					   "filename")
-		     (mew-syntax-get-file syntax))))
-      (if (null cdp)
-	  (setq cdp (read-string "Filename: " ocdp)))
-      (if (equal cdp "") (setq cdp nil)) ;; to clear
-      (mew-syntax-set-cdp syntax (mew-syntax-cdp-format cdp))
+	   (ctl (mew-syntax-get-ct syntax))
+	   (ct (mew-syntax-get-value ctl 'cap))
+	   (oldfile (or (mew-syntax-get-param (mew-syntax-get-cdp syntax)
+					      "filename")
+			(mew-syntax-get-file syntax)))
+	   (file (read-string "Filename: " oldfile)))
+      (if (string= file "")
+	  (mew-syntax-set-cdp syntax nil)
+	(mew-syntax-set-cdp syntax (mew-syntax-cdp-format ct file)))
       (mew-encode-syntax-print mew-encode-syntax))))
 
 (defun mew-attach-base64 ()
@@ -596,12 +557,11 @@ on '.' in attachments."
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (mew-syntax-get-value ctl 'cap))
-	   (case-fold-search t))
-	(if (or (string-match "^Message/" ct) (string-match "^Multipart/" ct))
-	    (message "Can't encode with base64 here")
-	  (mew-syntax-set-cte syntax mew-b64)
-	  (mew-encode-syntax-print mew-encode-syntax)))))
+	   (ct (mew-syntax-get-value ctl 'cap)))
+      (if (or (mew-ct-messagep ct) (mew-ct-multipartp ct))
+	  (message "Can't encode with base64 here")
+	(mew-syntax-set-cte syntax mew-b64)
+	(mew-encode-syntax-print mew-encode-syntax)))))
 
 (defun mew-attach-quoted-printable ()
   "Put the 'Q' mark to encode with Quoted-Printable in attachments"
@@ -611,12 +571,11 @@ on '.' in attachments."
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (mew-syntax-get-value ctl 'cap))
-	   (case-fold-search t))
-	(if (or (string-match "^Message/" ct) (string-match "^Multipart/" ct))
-	    (message "Can't encode with quoted-printable here")
-	  (mew-syntax-set-cte syntax mew-qp)
-	  (mew-encode-syntax-print mew-encode-syntax)))))
+	   (ct (mew-syntax-get-value ctl 'cap)))
+      (if (or (mew-ct-messagep ct) (mew-ct-multipartp ct))
+	  (message "Can't encode with quoted-printable here")
+	(mew-syntax-set-cte syntax mew-qp)
+	(mew-encode-syntax-print mew-encode-syntax)))))
 
 (defun mew-attach-gzip64 ()
   "Put the 'G' mark to encode with Gzip64 in attachments. This is 
@@ -628,11 +587,10 @@ is not effective other objects. For example, JPEG is already compressed."
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (ctl (mew-syntax-get-ct syntax))
-	   (ct (mew-syntax-get-value ctl 'cap))
-	   (case-fold-search t))
-      (if (or (string-match "^Message/" ct)
-	      (string-match "^Multipart/" ct)
-	      (not (mew-member-case-equal ct mew-mime-content-type-text-list)))
+	   (ct (mew-syntax-get-value ctl 'cap)))
+      (if (or (mew-ct-messagep ct)
+	      (mew-ct-multipartp ct)
+	      (not (mew-ct-linebasep ct)))
 	  ;; never use compress to types other than text/plain or 
 	  ;; application/postscript. Indeed, compression is not
 	  ;; effective to compressed data such as jpeg.
@@ -652,11 +610,11 @@ is not effective other objects. For example, JPEG is already compressed."
     (let* ((nums (mew-syntax-nums))
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (privacy (mew-syntax-get-privacy syntax))
-	   (pgp-sign (nth 1 (assoc 'pgp-signature mew-privacy-database))))
+	   (pgp-sign (mew-pcdb-ct (mew-pcdb-by-service 'pgp-signature))))
       (cond
        ((>= (length privacy) 2)
 	(message "Too many marks"))
-       ((member pgp-sign privacy)
+       ((member (car pgp-sign) privacy) ;; xxx hard coding
 	(message "Already marked."))
        (t
 	(mew-syntax-set-privacy 
@@ -677,11 +635,11 @@ Input decryptors' addresses."
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
 	   (privacy (mew-syntax-get-privacy syntax))
 	   (towhom (mew-header-parse-address-list mew-destination:-list))
-	   (pgp-enc (nth 1 (assoc 'pgp-encryption mew-privacy-database))))
+	   (pgp-enc (mew-pcdb-ct (mew-pcdb-by-service 'pgp-encryption))))
       (cond
        ((>= (length privacy) 2)
 	(message "Too many marks"))
-       ((member pgp-enc privacy)
+       ((member (car pgp-enc) privacy) ;; xxx hard coding
 	(message "Already marked."))
        (t
 	;; ask towhom before set privacy for C-g
@@ -709,7 +667,7 @@ Input decryptors' addresses."
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))	   
 	   (file (mew-syntax-get-file syntax))
 	   efile FILE)
-      (setq efile (if (equal subdir "") file (concat subdir file)))
+      (setq efile (if (string= subdir "") file (concat subdir file)))
       (setq FILE (expand-file-name efile attachdir))
       (find-file FILE)
       ;; buffer switched
@@ -721,7 +679,7 @@ Input decryptors' addresses."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1996-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
