@@ -2,18 +2,16 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: May 30, 1997
-;; Revised: Aug 30, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-complete-version "mew-complete.el version 0.04")
-
 (require 'mew)
 
 ;; hoping the functions here are free from marker problem
 ;; because it inserts any chars before \n. It is important
 ;; to use the position's color.
-(fset 'mew-complete-insert (symbol-function 'insert))
+(defalias 'mew-complete-insert 'insert)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -40,7 +38,7 @@
     (while (and (< (point-min) (point))	(looking-at mew-lwsp))
       (forward-line -1))
     (if (looking-at "\\([^:]*:\\)")
-	(mew-assoc-match (mew-match 1) switch 0)
+	(mew-field-get-func (mew-match 1) switch)
       nil))) ;; what a case reachs here?
   
 ;;
@@ -50,27 +48,25 @@
 (defvar mew-complete-candidates nil)
 
 (defun mew-complete-window-delete ()
-  (if (null mew-complete-window-config)
-      ()
-    ;; mew-complete-window-config remains when the last completion  
+  (when (mew-ainfo-get-win-cfg)
+    ;; (mew-ainfo-get-win-cfg) remains when the last completion  
     ;; finished with multiple candidates.
     ;; (e.g. foo<RET> when foo and foobar are displayed.)
     ;; In this case, this function is called in another
     ;; completion thread but setting window configuration is not
     ;; desired. If we set window configuration with the old
-    ;; mew-complete-window-config, the cursor jumps to mini buffer.
+    ;; (mew-ainfo-get-win-cfg), the cursor jumps to mini buffer.
     ;; This was a stupid bug of Mew. So, let's see if the complete
     ;; buffer is displayed or not.
     (if (get-buffer-window mew-buffer-completions)
-	(set-window-configuration mew-complete-window-config))
-    (setq mew-complete-window-config nil))
-  (and (get-buffer mew-buffer-completions)
-       (kill-buffer mew-buffer-completions))
+	(set-window-configuration (mew-ainfo-get-win-cfg)))
+    (mew-ainfo-set-win-cfg nil))
+  (mew-remove-buffer mew-buffer-completions)
   (setq mew-complete-candidates nil))
 
 (defun mew-complete-window-show (all)
-  (or mew-complete-window-config
-      (setq mew-complete-window-config (current-window-configuration)))
+  (unless (mew-ainfo-get-win-cfg)
+    (mew-ainfo-set-win-cfg (current-window-configuration)))
   (if (and (get-buffer-window mew-buffer-completions)
 	   (equal mew-complete-candidates all))
       (let ((win (get-buffer-window mew-buffer-completions)))
@@ -105,7 +101,7 @@ is inserted before the cursor, the short name is expanded to its address."
       (mew-complete-field)
     (let ((func (mew-draft-on-value-p mew-field-completion-switch)))
       (if func 
-	  (funcall (cdr func))
+	  (funcall func)
 	(tab-to-tab-stop))))) ;; default keybinding
 
 (defun mew-complete-field ()
@@ -116,7 +112,7 @@ is inserted before the cursor, the short name is expanded to its address."
 	(mew-complete-window-show mew-fields)
       (mew-complete
        word
-       (mapcar (function (lambda (x) (list (concat (capitalize x) " "))))
+       (mapcar (function (lambda (x) (list (concat (mew-capitalize x) " "))))
 	       mew-fields)
        "field"
        nil))))
@@ -124,11 +120,10 @@ is inserted before the cursor, the short name is expanded to its address."
 (defun mew-complete-folder ()
   "Folder complete function for Fcc:."
   (interactive)
-  (let ((word (mew-delete-backward-char)))
+  (let ((word (mew-delete-backward-char))
+	(completion-ignore-case mew-complete-folder-ignore-case))
     (if (null word)
-	(if mew-use-imap
-	    (mew-complete-window-show (list "+" "=" "%"))
-	  (mew-complete-window-show (list "+" "=")))
+	(mew-complete-window-show (list "+" "="))
       (mew-complete word mew-folder-alist "folder" nil))))
 
 (defun mew-complete-address ()
@@ -136,7 +131,8 @@ is inserted before the cursor, the short name is expanded to its address."
 First alias key is completed. When completed solely or the @ character
 is inserted before the cursor, the short name is expanded to its address."
   (interactive)
-  (let ((word (mew-delete-backward-char)))
+  (let ((word (mew-delete-backward-char))
+    	(completion-ignore-case mew-complete-address-ignore-case))
     (if (null word)
 	(tab-to-tab-stop)
       (if (string-match "@." word)
@@ -146,14 +142,27 @@ is inserted before the cursor, the short name is expanded to its address."
 	 (function mew-addrbook-alias-get) 
 	 (function mew-addrbook-alias-hit))))))
 
-(defun mew-complete-config ()
-  "Complete function for Config:."
+(defun mew-complete-newsgroups ()
   (interactive)
-  (let ((word (or (mew-delete-value ",") "")))
+  (let ((word (mew-delete-backward-char))
+    	(completion-ignore-case mew-complete-address-ignore-case))
+    (if (null word)
+	(tab-to-tab-stop)
+      (mew-complete
+       word
+       mew-newsgroups-alist
+       "Newsgroups"
+       nil))))
+
+(defun mew-complete-case ()
+  "Complete function for cases."
+  (interactive)
+  (let ((word (or (mew-delete-backward-char) ""))
+    	(completion-ignore-case mew-complete-case-ignore-case))
     (mew-complete
      word
-     (mew-slide-pair mew-config-list)
-     "mew-config-list"
+     (mapcar (function (lambda (x) (cons x x))) mew-config-cases)
+     "Case"
      nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -166,7 +175,7 @@ is inserted before the cursor, the short name is expanded to its address."
   (interactive)
   (let ((func (mew-draft-on-value-p mew-field-circular-completion-switch)))
     (if func
-	(funcall (cdr func))
+	(funcall func)
       (message "No circular completion here"))))
 
 (defun mew-circular-complete-domain ()
@@ -176,15 +185,16 @@ mew-mail-domain-list is inserted. If exists, the next value of
 mew-mail-domain-list concerned with the string between @ and 
 the cursor is inserted."
   (interactive)
-  (let ((word (mew-delete-backward-char "@")))
+  (let ((word (mew-delete-backward-char "@"))
+	(completion-ignore-case mew-circular-complete-domain-ignore-case))
     (cond
-     ((equal word nil) ;; @ doesn't exist.
+     ((eq word nil) ;; @ doesn't exist.
       (if (null mew-mail-domain-list)
 	  (message "For domain circular completion, set mew-mail-domain-list")
 	(mew-complete-insert "@")
 	(mew-complete-insert (car mew-mail-domain-list))
 	(mew-complete-window-delete)))
-     ((equal word t) ;; just after @
+     ((eq word t) ;; just after @
       (if (null mew-mail-domain-list)
 	  (message "For domain circular completion, set mew-mail-domain-list")
 	(mew-complete-insert (car mew-mail-domain-list))
@@ -217,11 +227,6 @@ the cursor is inserted."
   (interactive)
   (mew-circular-complete "from" mew-from-list "mew-from-list"))
 
-(defun mew-circular-complete-config ()
-  "Circular complete function for Config:."
-  (interactive)
-  (mew-circular-complete "config" mew-config-list "mew-config-list" ","))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Expansion : M-C-i
@@ -232,11 +237,11 @@ the cursor is inserted."
   (interactive)
   (let ((func (mew-draft-on-value-p mew-field-expansion-switch)))
     (if func
-	(funcall (cdr func))
+	(funcall func)
       (message "No expansion here"))))
 
 (defun mew-expand-address ()
-  "Address expansion fuction for To:, Cc:, etc.
+  "Address expansion function for To:, Cc:, etc.
 'user@domain' will be expands 'name <user@domain>' if
 the name exists."
   (interactive)
@@ -310,7 +315,7 @@ the name exists."
 ;;; Hart function for completions
 ;;;
 
-(fset 'mew-complete-hit (symbol-function 'assoc))
+(defalias 'mew-complete-hit 'assoc)
 
 (defun mew-complete-get (key alist)
   (cdr (mew-complete-hit key alist)))
@@ -339,12 +344,12 @@ the name exists."
       (mew-complete-insert (funcall fget subkey ALIST)) ;; use cdr
       (mew-complete-window-delete))
      ;; just one candidate
-     ((equal 1 (length all))
+     ((= 1 (length all))
       (mew-complete-insert cmp)
-      (mew-complete-window-delete)
       (if (window-minibuffer-p (get-buffer-window (current-buffer)))
 	  (mew-temp-minibuffer-message " [Sole completion]")
-	(message "Sole completion")))
+	(message "Sole completion"))
+      (mew-complete-window-delete))
      ;; two or more candidates
      ((stringp cmp) ;; (length all) > 1
       (mew-complete-insert cmp)
@@ -373,7 +378,7 @@ the name exists."
       (goto-char (point-max))
       (mew-complete-insert m))
     (let ((inhibit-quit t))
-      (sit-for 2)
+      (mew-let-user-read)
       (delete-region savemax (point-max))
       (if quit-flag (setq quit-flag nil	unread-command-events 7)))))
 
@@ -382,7 +387,7 @@ the name exists."
 ;;
 
 (defun mew-delete-backward-char (&optional here)
-  "Delete appropriate preceeding word and return it."
+  "Delete appropriate preceding word and return it."
   (interactive)
   (let ((case-fold-search t)
         (start nil)
@@ -431,7 +436,7 @@ the name exists."
   (let ((pos (point)))
     (beginning-of-line)
     (prog1
-	(capitalize (mew-buffer-substring (point) pos))
+	(mew-capitalize (mew-buffer-substring (point) pos))
       (delete-region (point) pos))))
 
 (defun mew-delete-value (&optional here)
@@ -459,11 +464,12 @@ the name exists."
 ;;
 
 (defun mew-slide-pair (x)
-  (let ((ret nil)
+  (let ((len (length x))
+	(ret nil)
 	(first (car x)))
     (cond 
-     ((eq x 0) nil)
-     ((eq x 1) (cons first first))
+     ((= len 0) nil)
+     ((= len 1) (list (cons first first)))
      (t
       (while (cdr x)
 	(setq ret (cons (cons (nth 0 x) (nth 1 x)) ret))
@@ -475,7 +481,7 @@ the name exists."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1997-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
