@@ -2,14 +2,18 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-demo-version "mew-demo.el version 0.11")
-
 (require 'mew-vars)
 
+(defvar mew-logo nil)
+
+(defconst mew-icon-mew "Mew.png")
+(defconst mew-icon-mew-mono "Mew.xbm")
+(defconst mew-icon-mew-mule-bitmap-image "Mew.img")
+
 (defconst mew-hello-message 
 "
 
@@ -19,7 +23,7 @@ Mew -- Messaging in the Emacs World
 
 %s
 
-Copyright (C) 1994-1998 Kazu Yamamoto
+Copyright (C) 1994-2001 Kazu Yamamoto
 
 Please send comments to Kazu@Mew.org.
 
@@ -51,34 +55,43 @@ Please send comments to Kazu@Mew.org.
 )
 
 (defun mew-hello ()
-  (mew-window-configure (get-buffer-create mew-buffer-hello) '(1 0))
-  (let (left-margin fill-column ext)
+  (let ((time mew-demo-sit-for)
+	(left-margin 0)
+	(fill-column (window-width))
+	indent)
+    (if (not (integerp time)) (setq time 0))
     (mew-erase-buffer)
-    (setq left-margin 0)
-    (setq fill-column (window-width))
     (insert (format mew-hello-message mew-version))
     (center-region (point-min) (point-max))
     (cond
-     (mew-xemacs-p
-      (goto-char (point-min))
-      (setq ext (make-extent (point-min) (point-max)))
-;      (set-extent-face ext 'bold-italic) ;; not work for 14 dot...
-      (set-extent-face ext 'bold)
+     ((and mew-xemacs-p window-system)
       (goto-char (point-max))
       (cond
-       ((and window-system mew-demo-picture
+       ((and mew-demo-picture
 	     (valid-image-instantiator-format-p 'png))
-	(setq mew-logo
-	      (make-glyph (vector 'png ':file 
-				  (expand-file-name mew-icon-mew
-						    mew-icon-directory))))
+	(mew-flet
+	 (setq mew-logo
+	       (make-glyph (vector 'png ':file 
+				   (expand-file-name mew-icon-mew
+						     mew-icon-directory)))))
 	(indent-to (startup-center-spaces mew-logo))
-	(set-extent-begin-glyph (make-extent (point) (point)) mew-logo)
+	(set-extent-begin-glyph (mew-overlay-make (point) (point)) mew-logo)
 	(goto-char (point-min))
-	)
-       )
-      (sit-for 0) ;; to redraw
-      )
+	(mew-redraw time))
+       (t
+	(mew-hello-text time))))
+     ((and window-system
+	   (fboundp 'image-type-available-p)
+	   (image-type-available-p 'png))
+      (goto-char (point-max))
+      (setq mew-logo (create-image 
+		      (expand-file-name mew-icon-mew mew-icon-directory)))
+      (setq indent (/ (- (frame-width)
+			 (truncate (car (image-size mew-logo)))) 2))
+      (insert (make-string indent 32))
+      (insert-image mew-logo)
+      (goto-char (point-min))
+      (mew-redraw time))
      ((and (featurep 'bitmap) mew-demo-picture)
       (or
        (condition-case nil
@@ -87,24 +100,28 @@ Please send comments to Kazu@Mew.org.
 			      mew-icon-directory))
 	 (file-error nil))
        (condition-case nil
-	   (progn
+	   (let ((buf (generate-new-buffer mew-buffer-prefix)))
 	     (save-excursion
-	       (mew-set-buffer-tmp)
+	       (set-buffer buf)
+	       (mew-erase-buffer)
 	       (insert-file-contents
 		(expand-file-name mew-icon-mew-mono mew-icon-directory)))
-	     (bitmap-insert-xbm-buffer mew-buffer-tmp))
+	     (bitmap-insert-xbm-buffer buf)
+	     (mew-remove-buffer buf))
 	 (file-error nil)))
       (center-region (point-min) (point-max))
-      (sit-for 0)) ;; to redraw
+      (mew-redraw time))
      (t
-      (insert "/\\\\ - \\\\/")
-      (center-line)
-      (end-of-line)
-      (insert (make-string (1- (- (window-width) (current-column))) ?\ ))
-      (end-of-line)
-      (sit-for 0) ;; to redraw
-      (if mew-demo (mew-demo)))
-     )))
+      (mew-hello-text time)))))
+
+(defun mew-hello-text (time)
+  (insert "/\\\\ - \\\\/")
+  (center-line)
+  (end-of-line)
+  (insert (make-string (1- (- (window-width) (current-column))) ?\ ))
+  (end-of-line)
+  (mew-redraw time)
+  (if (and mew-demo window-system) (mew-demo)))
 
 (defun mew-demo (&optional string)
   (let* ((list (or string mew-demo-string))
@@ -114,12 +131,10 @@ Please send comments to Kazu@Mew.org.
 	 (pre (make-string pl 32))
 	 (suf (make-string (1- (- (- wl pl) ul)) 32)))
     (save-window-excursion
-      (select-window (get-buffer-window (get-buffer mew-buffer-hello)))
       (while list
 	(mew-demo-print (car list) pre suf)
 	(mew-demo-loop)
-	(setq list (cdr list)))
-      )))
+	(setq list (cdr list))))))
 
 (defun mew-demo-print (string prefix suffix)
   (goto-char (point-max))
@@ -129,13 +144,13 @@ Please send comments to Kazu@Mew.org.
   (insert (concat prefix string suffix)))
 
 (defun mew-demo-loop ()
-  (sit-for 0.02))
+  (mew-redraw 0.02))
 
 (provide 'mew-demo)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
