@@ -2,47 +2,23 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Nov 13, 1996
-;; Revised: Aug 30, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-ext-version "mew-ext.el version 0.21")
-
 (require 'mew)
 (eval-when-compile
   (cond
-   ((mew-which-el "efs" load-path)
+   ((mew-which-el "efs")
     (require 'efs))
-   ((mew-which-el "ange-ftp" load-path)
+   ((mew-which-el "ange-ftp")
     (require 'ange-ftp))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; customize variables
+;;; Variables for encoding
 ;;;
 
-(defvar mew-ext-prog-url "netscape")
-(defvar mew-ext-prog-url-args '("-install"))
-
-;; If you want to use w3.el instead of "netscape", put the 
-;; following in .emacs.
-;;(setq mew-ext-prog-url "w3")
-;;(setq mew-ext-prog-url-args nil)
-
-;; If you want to use lynx instead of "netscape", put the 
-;; following in .emacs.
-;;(setq mew-ext-prog-url "kterm")
-;;(setq mew-ext-prog-url-args '("-e" "lynx" "-color"))
-
-(defvar mew-ext-anon-ftp-method 'ftp
-  "A method to get the message body for access-type=anon-ftp.
-If 'ftp is specified, ange-ftp or efs is used. If 'http is specified,
-w3 is used.")
-
-;;
-;; encode
-;;
-
 (defvar mew-ext-default-access-type "anon-ftp")
 
 (defvar mew-ext-ftp-server-list 
@@ -56,19 +32,29 @@ w3 is used.")
     ("mail-server" . mew-ext-encode-mail-server)
     ("url"         . mew-ext-encode-url)))
 
-;;
-;; decode
-;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;;  Variables for decoding
+;;;
 
 (defvar mew-ext-switch
-  '(("ftp"         . mew-ext-ftp)
-    ("tftp"        . mew-ext-tftp)
-    ("anon-ftp"    . mew-ext-anon-ftp)
-    ("mail-server" . mew-ext-mail-server)
-    ("url"         . mew-ext-url))) ;; RFC2017
+  '(("ftp"         mew-ext-ftp         mew-ext-ftp-ext)
+    ("tftp"        mew-ext-tftp        nil)
+    ("anon-ftp"    mew-ext-anon-ftp    mew-ext-anon-ftp-ext)
+    ("mail-server" mew-ext-mail-server mew-ext-mail-server-ext)
+    ("url"         mew-ext-url         mew-ext-url-ext))) ;; RFC 2017
+
+(defsubst mew-ext-get-func (acc-type)
+  (nth 1 (mew-assoc-match acc-type mew-ext-switch 0)))
+
+(defsubst mew-ext-get-func-ext (acc-type)
+  (nth 2 (mew-assoc-match acc-type mew-ext-switch 0)))
 
 (defvar mew-ext-include-switch 
-  '(("local-file" . mew-ext-include-local-file)))
+  '(("local-file" mew-ext-include-local-file)))
+
+(defsubst mew-ext-include-get-func (acc-type)
+  (nth 1 (mew-assoc-case-equal acc-type mew-ext-include-switch 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -76,7 +62,7 @@ w3 is used.")
 ;;;
 
 (cond
- ((mew-which-el "efs" load-path)
+ ((mew-which-el "efs")
   (defun mew-ext-file-name-completion (file path)
     (require 'efs)
     (let ((efs-tmp-name-template mew-temp-file))
@@ -98,7 +84,7 @@ w3 is used.")
 	  (parsed (efs-ftp-path remote)))
       (efs-copy-file-internal remote parsed local nil 
 			      nil nil nil nil t 'image))))
- ((mew-which-el "ange-ftp" load-path)
+ ((mew-which-el "ange-ftp")
   (defun mew-ext-file-name-completion (file path)
     (require 'ange-ftp)
     (let ((ange-ftp-tmp-name-template mew-temp-file))
@@ -117,8 +103,7 @@ w3 is used.")
     (require 'ange-ftp)
     (let ((ange-ftp-tmp-name-template mew-temp-file)
 	  (ange-ftp-generate-anonymous-password passwd))
-      (ange-ftp-copy-file-internal remote local t nil nil nil t))))
- )
+      (ange-ftp-copy-file-internal remote local t nil nil nil t)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -134,10 +119,10 @@ w3 is used.")
 	   (attachdir (mew-attachdir))
 	   file filepath ct)
       ;; attachdir / {subdir/} dir
-      (if (not (equal subdir "")) 
+      (if (not (string= subdir "")) 
 	  (setq attachdir (expand-file-name subdir attachdir)))
       ;; attachdir / file
-      (setq filepath (mew-random-filename attachdir mew-ext-suffix))
+      (setq filepath (mew-random-filename attachdir 1 nil mew-ext-suffix))
       (if (file-exists-p filepath)
 	  (message "Could not make a file for external-body, sorry.")
 	(setq file (file-name-nondirectory filepath))
@@ -155,11 +140,13 @@ w3 is used.")
   (format "<%s.%s.%s@%s>" (nth 0 (current-time)) (nth 1 (current-time)) 
 	  (emacs-pid) (system-name)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun mew-ext-encode (filename)
-  (let (buf ret access-type ct name)
+  (let ((buf (generate-new-buffer mew-buffer-prefix))
+	ret access-type ct name)
     (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-ext))
-      (setq buf (current-buffer))
+      (set-buffer buf)
       (mew-erase-buffer)
       ;;content-header
       (setq access-type (mew-input-general
@@ -178,9 +165,9 @@ w3 is used.")
 	(setq name (file-name-nondirectory
 		    (mew-chop (mew-syntax-get-param ret "name"))))
 	;; name is quoted
-	(if (equal name "")
+	(if (string= name "")
 	    (setq ct mew-ct-apo)
-	  (setq ct (capitalize (mew-attr-get-ct (mew-attr-by-file name)))))))
+	  (setq ct (mew-ctdb-ct (mew-ctdb-by-file name))))))
       (setq ct (mew-input-type "Type for %s (%s): " name ct
 			       mew-mime-content-type-list))
       (mew-header-insert mew-ct: ct)
@@ -192,9 +179,11 @@ w3 is used.")
 	(insert (read-string "Input message to the mail-server: "))
 	(insert "\n"))
       (write-file filename))
-    (kill-buffer buf)
+    (mew-remove-buffer buf)
     (cons mew-ct-ext (cons (list "access-type" access-type) ret))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun mew-ext-encode-ftp ()
   ;; "name" "site" "directory" "mode"
   (let ((mew-ext-host (mew-input-general 
@@ -207,7 +196,7 @@ w3 is used.")
 	mew-ext-user path dir file ret)
     (setq ret (list (list "site" mew-ext-host)))
     (setq mew-ext-user (read-string (format "User name at %s: " mew-ext-host)
-				(user-login-name)))
+				    (mew-user)))
     (setq path (mew-input-rfile "Filename :"))
     (setq file (file-name-nondirectory path))
     (setq dir (file-name-directory path))
@@ -221,10 +210,14 @@ w3 is used.")
       (setq ret (cons (list "name" file) ret))))
     ret))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun mew-ext-encode-tftp ()
   ;; xxx not yet
   )
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun mew-ext-encode-anon-ftp ()
   ;; "name" "site" "directory" "mode"
   (let ((mew-ext-user "anonymous")
@@ -250,11 +243,15 @@ w3 is used.")
       (setq ret (cons (list "name" file) ret))))
     ret))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun mew-ext-encode-local-file ()
   ;; "name" "site"
   (let ((file (mew-draft-input-file-name "File name: ")))
     (list (list "name" (expand-file-name file)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun mew-ext-encode-mail-server ()
   ;; "server" "subject"
   (let (server subject)
@@ -262,6 +259,8 @@ w3 is used.")
     (setq subject (read-string (concat mew-subj: " ")))
     (list (list "server" server)
 	  (list "subject" subject))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
 (defun mew-ext-encode-url ()
   ;; "url"
@@ -277,22 +276,28 @@ w3 is used.")
 ;; exclude
 ;;
 
-(defun mew-mime-external-body (begin end params &optional execute)
+(defun mew-mime-external-body (cache begin end &optional params)
   ;; message-buffer
   (let* ((access-type (mew-syntax-get-param params "access-type"))
-	 (program (cdr (mew-assoc-match access-type mew-ext-switch 0))))
-    ;; xxx expire
-    (if (and (symbolp program) (fboundp program))
-	(funcall program begin end params execute))))
+	 (func (mew-ext-get-func access-type)))
+    (if (and (symbolp func) (fboundp func))
+	(funcall func cache begin end params))))
 
-(defun mew-ext-ftp (begin end params execute)
+(defun mew-mime-external-body-ext (cache begin end &optional params)
+  (let* ((access-type (mew-syntax-get-param params "access-type"))
+	 (func (mew-ext-get-func-ext access-type)))
+    (if (and (symbolp func) (fboundp func))
+	(funcall func cache begin end params))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-ext-ftp (cache begin end &optional params)
   (mew-elet
    (let* ((site (mew-syntax-get-param params "site"))
 	  (directory (mew-syntax-get-param params "directory"))
 	  (name (mew-syntax-get-param params "name"))
 	  (size (mew-syntax-get-param params "size"))
-	  (getit t) (username "")
-	  filepath localfile lfname remotefile)
+	  filepath)
      (if directory
 	 (setq filepath (concat (file-name-as-directory directory) name))
        (setq filepath name))
@@ -308,36 +313,50 @@ w3 is used.")
 	     " #          #    #       \n"
 	     "\n\n")
      (insert "You can get the message content by FTP\n\n")
-     (mew-insert "SITE:\t%s\n" site)
-     (mew-insert "FILE:\t%s\n" filepath)
-     (mew-insert "SIZE:\t%s bytes\n" size)
-     (if (null execute)
-	 (insert "\nTo get this file, type "
-		 (substitute-command-keys
-		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
-       (setq username (read-string (format "User name at %s: " site)
-				   (user-login-name)))
-       (setq remotefile (format "/%s@%s:%s" username site filepath))
-       (setq localfile (mew-summary-input-file-name "Save to: " name))
-       (setq lfname (file-name-nondirectory localfile))
-       (if (file-exists-p localfile)
-	   (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
-	       (delete-file localfile)
-	     (setq getit nil)
-	     (message "The file wasn't retrieved")))
-       (if getit (mew-ext-copy-file-internal remotefile localfile nil))))))
+     (mew-insert "Site:\t%s\n" site)
+     (mew-insert "File:\t%s\n" filepath)
+     (mew-insert "Size:\t%s bytes\n" size)
+     (insert "\n")
+     (mew-insert-manual
+      "To get this file, type "
+      "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'.\n"))))
 
-(defun mew-ext-tftp (begin end params execute)
+(defun mew-ext-ftp-ext (cache begin end &optional params)
+  (let* ((site (mew-syntax-get-param params "site"))
+	 (directory (mew-syntax-get-param params "directory"))
+	 (name (mew-syntax-get-param params "name"))
+	 (getit t) (username "")
+	 filepath localfile lfname remotefile)
+    (if directory
+	(setq filepath (concat (file-name-as-directory directory) name))
+      (setq filepath name))
+    (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
+	(setq filepath (substring filepath 2 (match-end 0))))
+    (setq username (read-string (format "User name at %s: " site) (mew-user)))
+    (setq remotefile (format "/%s@%s:%s" username site filepath))
+    (setq localfile (mew-summary-input-file-name "Save to: " name))
+    (setq lfname (file-name-nondirectory localfile))
+    (if (file-exists-p localfile)
+	(if (y-or-n-p (format "%s exists. Overwrite? " lfname))
+	    (delete-file localfile)
+	  (setq getit nil)
+	  (message "The file wasn't retrieved")))
+    (if getit (mew-ext-copy-file-internal remotefile localfile nil))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-ext-tftp (cache begin end &optional params)
   (message "access-type TFTP is not supported yet"))
 
-(defun mew-ext-anon-ftp (begin end params execute)
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-ext-anon-ftp (cache begin end &optional params)
   (mew-elet
    (let* ((site (mew-syntax-get-param params "site"))
 	  (directory (mew-syntax-get-param params "directory"))
 	  (name (mew-syntax-get-param params "name"))
 	  (size (mew-syntax-get-param params "size"))
-	  (getit t)
-	  filepath localfile lfname remotefile url)
+	  filepath)
      (if directory
 	 (setq filepath (concat (file-name-as-directory directory) name))
        (setq filepath name))
@@ -354,39 +373,51 @@ w3 is used.")
 	     " #          #    #       \n"
 	     "\n\n")
      (insert "You can get the message content by FTP\n\n")
-     (mew-insert "SITE:\t%s\n" site)
-     (mew-insert "FILE:\t%s\n" filepath)
-     (mew-insert "SIZE:\t%s bytes\n" size)
-     (if (null execute)
-	 (insert "\nTo get this file, type "
-		 (substitute-command-keys
-		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
-       (setq remotefile (format "/%s@%s:%s" "anonymous" site filepath))
-       (if (string-match "^[/\:]" filepath)
-	   (setq url (format "ftp://%s%s" site filepath))
-	 (setq url (format "ftp://%s/%s" site filepath)))
-       (setq localfile (mew-summary-input-file-name "Save to: " name))
-       (setq lfname (file-name-nondirectory localfile))
-       (if (file-exists-p localfile)
-	   (if (y-or-n-p (format "%s exists. Overwrite? " lfname))
-	       (delete-file localfile)
-	     (setq getit nil)
-	     (message "The file wasn't retrieved")))
-       (if (not getit)
-	   ()
-	 (cond
-	  ((eq mew-ext-anon-ftp-method 'ftp)
-	   (mew-ext-copy-file-internal remotefile localfile mew-mail-address))
-	  ((eq mew-ext-anon-ftp-method 'http)
-	   (require 'w3)
-	   (w3-fetch url))))))))
+     (mew-insert "Site:\t%s\n" site)
+     (mew-insert "File:\t%s\n" filepath)
+     (mew-insert "Size:\t%s bytes\n" size)
+     (insert "\n")
+     (mew-insert-manual
+      "To get this file, type "
+      "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'.\n"))))
 
-(defun mew-ext-mail-server (begin end params execute)
+(defun mew-ext-anon-ftp-ext (cache begin end &optional params)
+  (let* ((site (mew-syntax-get-param params "site"))
+	 (directory (mew-syntax-get-param params "directory"))
+	 (name (mew-syntax-get-param params "name"))
+	 (getit t)
+	 filepath localfile lfname remotefile url)
+    (if directory
+	(setq filepath (concat (file-name-as-directory directory) name))
+      (setq filepath name))
+    (if (and filepath (string-match "^[A-Za-z]:/.+" filepath)) ;; drive letter
+	(setq filepath (substring filepath 2 (match-end 0))))
+    (setq remotefile (format "/%s@%s:%s" "anonymous" site filepath))
+    (if (string-match "^[/\:]" filepath)
+	(setq url (format "ftp://%s%s" site filepath))
+      (setq url (format "ftp://%s/%s" site filepath)))
+    (setq localfile (mew-summary-input-file-name "Save to: " name))
+    (setq lfname (file-name-nondirectory localfile))
+    (if (file-exists-p localfile)
+	(if (y-or-n-p (format "%s exists. Overwrite? " lfname))
+	    (delete-file localfile)
+	  (setq getit nil)
+	  (message "The file wasn't retrieved")))
+    (when getit
+      (cond
+       ((eq mew-ext-anon-ftp-method 'ftp)
+	(mew-ext-copy-file-internal
+	 remotefile localfile (mew-mail-address)))
+       ((eq mew-ext-anon-ftp-method 'http)
+	(require 'w3)
+	(w3-fetch url))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-ext-mail-server (cache begin end &optional params)
   (mew-elet
-   (let ((server (mew-syntax-get-param params "server"))
-	 (subject (mew-syntax-get-param params "subject"))
-	 (size (mew-syntax-get-param params "size"))
-	 (start nil))
+   (let* ((server (mew-syntax-get-param params "server"))
+	  (size (mew-syntax-get-param params "size")))
      (mew-erase-buffer)
      (insert " #     #    #      ###   #\n"
 	     " ##   ##   # #      #    #\n"
@@ -397,34 +428,49 @@ w3 is used.")
 	     " #     # #     #   ###   #######\n"
 	     "\n\n")
      (insert "You can get the message by e-mail\n\n")
-     (mew-insert "SERVER:\t\t%s\n" server)
-     (mew-insert "SIZE:\t%s bytes\n" size)
-     (save-excursion
-       (set-buffer (mew-current-get 'cache))
-       (save-restriction
-	 (narrow-to-region begin end)
-	 (goto-char (point-min))
-	 ;; find a phantom body (in RFC1521)
-	 (re-search-forward "^$" nil t)
-	 (forward-line)
-	 (setq start (point))))
+     (mew-insert "Server:\t\t%s\n" server)
+     (mew-insert "Size:\t%s bytes\n" size)
+     (insert "\n")
      ;; pickd up source from 'mew-send
-     (if (null execute)
-	 (insert "\nTo send this mail, type "
-		 (substitute-command-keys
-		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
-       (mew-summary-send server nil subject)
-       (goto-char (point-max))
-       (insert-buffer-substring (mew-current-get 'cache) start end)
-       (mew-draft-make-message)
-       (if (y-or-n-p "Send this message? ")
-	   (mew-draft-real-send-letter))))))
+     (mew-insert-manual
+      "To send this mail, type "
+      "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'.\n"))))
 
-(defun mew-ext-url (begin end params execute)
+(defun mew-ext-mail-server-ext (cache begin end &optional params)
+  (let* ((server (mew-syntax-get-param params "server"))
+	 (subject (mew-syntax-get-param params "subject"))
+	 (syntax (mew-ext-decode-message-header cache begin end))
+	 (start (mew-syntax-get-begin syntax)))
+    (mew-summary-send server nil subject)
+    (goto-char (point-max))
+    (mew-insert-buffer-substring cache start end)
+    (mew-draft-make-message)))
+
+(defun mew-ext-decode-message-header (cache begin end)
+  (let (syntax start)
+    (save-excursion
+      (set-buffer cache)
+      (save-restriction
+	(narrow-to-region begin end)
+	(goto-char (point-min))
+	(setq syntax (mew-decode-mime-header))
+	(setq start (point)))
+      (mew-syntax-set-key syntax 'phantom)
+      (mew-syntax-set-begin syntax start)
+      (mew-syntax-set-end syntax end)
+      syntax)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun mew-ext-url (cache begin end &optional params)
   (mew-elet
-   (let ((url (mew-syntax-get-param params "url"))
-	 (size (mew-syntax-get-param params "size"))
-	 (process-connection-type mew-connection-type1))
+   (let* ((url (mew-syntax-get-param params "url"))
+	  (size (mew-syntax-get-param params "size"))
+	  (process-connection-type mew-connection-type1)
+	  (syntax (mew-ext-decode-message-header cache begin end))
+	  (mct (car (mew-syntax-get-ct syntax)))
+	  (prog-list (mew-ext-url-by-ct mct))
+	  (doc (mew-ext-url-get-doc prog-list)))
      (mew-erase-buffer)
      (insert "#     # ######  #\n"
 	     "#     # #     # #\n"
@@ -435,28 +481,45 @@ w3 is used.")
 	     " #####  #     # #######\n"
 	     "\n\n")
      (mew-insert "URL:\t\t%s\n" url)
-     (mew-insert "SIZE:\t%s bytes\n" size)
-     (insert (format "Browser:\t%s\n"
-		     (cond ((and (symbolp mew-ext-prog-url)
-				 (fboundp mew-ext-prog-url))
-			    (symbol-name mew-ext-prog-url))
-			   ((stringp mew-ext-prog-url) mew-ext-prog-url)
-			   (t "none"))))
-     (if (null execute)
-	 (insert "\nTo show this URL, type "
-		 (substitute-command-keys
-		  "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'."))
-       (cond
-	((and (symbolp mew-ext-prog-url) (fboundp mew-ext-prog-url))
-	 (funcall mew-ext-prog-url url))
-	((equal mew-ext-prog-url "w3")
-	 (require 'w3)
-	 (w3-fetch-other-frame url))
-	(t
-	 (apply (function start-process)
-		(format "*mew %s*" mew-ext-prog-url)
-		mew-buffer-tmp mew-ext-prog-url 
-		(append mew-ext-prog-url-args (list url)))))))))
+     (mew-insert "Content-Type:\t%s\n" mct)
+     (mew-insert "Size:\t%s bytes\n" size)
+     (mew-insert "Operation:\t%s\n" (or doc "not defined"))
+     (insert "\n")
+     (mew-insert-manual
+      "To operate this URL, type "
+      "'\\<mew-summary-mode-map>\\[mew-summary-execute-external]'.\n"))))
+
+(defun mew-ext-url-ext (cache begin end &optional params)
+  (let* ((url (mew-syntax-get-param params "url"))
+	 (process-connection-type mew-connection-type1)
+	 (syntax (mew-ext-decode-message-header cache begin end))
+	 (mct (car (mew-syntax-get-ct syntax)))
+	 (prog-list (mew-ext-url-by-ct mct))
+	 (prog (mew-ext-url-get-prog prog-list))
+	 (args (mew-ext-url-get-args prog-list))
+	 pnm)
+    (cond
+     ((and (symbolp prog) (fboundp prog))
+      (message "Call function: (%s %s %s)" prog args url)
+      (funcall prog args url))
+     ((stringp prog)
+      (setq args (append args (list url)))
+      (message "Running command: %s"
+	       (mapconcat (function append) (append (list prog) args) " "))
+      (save-excursion ;; xxx this is necessary in other places
+	(setq pnm (format "*Mew %s*" prog))
+	(apply (function start-process) pnm nil prog args)))
+     (t (message "Program list error.")))))
+
+(defun mew-ext-url-show-by-w3 (dummy url)
+  (require 'w3)
+  (w3-fetch-other-frame url))
+
+(defun mew-ext-url-fetch-by-w3 (dummy url)
+  (require 'w3)
+  (w3-fetch url))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;
 ;; include
@@ -472,7 +535,7 @@ w3 is used.")
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1996-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
