@@ -2,15 +2,12 @@
 
 ;; Author:  Yoshinari NOMURA <nom@csce.kyushu-u.ac.jp>
 ;; Created: Nov 29, 1994
-;; Revised: Aug 30, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-fib-version "mew-fib.el version 0.12")
-
 (require 'mew-func)
 
-(defvar mew-fib-item-file "~/.mew-fib")
 (defvar mew-fib-item-alist nil)
 
 (defun mew-fib-split (str)
@@ -25,26 +22,29 @@
     (reverse ret)))
 
 (defun mew-fib-make-alist ()
-  (save-excursion
-    (let ((fib-file (expand-file-name mew-fib-item-file))
-	  item val ret tmp-val)
-      (setq mew-fib-item-alist nil)
-      (mew-set-buffer-tmp)
+  (let ((fib-file (expand-file-name mew-fib-item-file))
+	(buf (generate-new-buffer mew-buffer-prefix))
+	item val ret tmp-val)
+    (setq mew-fib-item-alist nil)
+    (save-excursion
+      (set-buffer buf)
+      (mew-erase-buffer)
       (if (file-exists-p fib-file)
 	  (insert-file-contents fib-file))
       (goto-char (point-min))
       (delete-matching-lines "^[ \t]*[;#%]")
       (while (re-search-forward "^\\([^:]+\\):[ \t]*\\(.*\\)$" nil t)
-	(setq item    (buffer-substring (match-beginning 1) (match-end 1))
-	      tmp-val (buffer-substring (match-beginning 2) (match-end 2))
+	(setq item    (mew-buffer-substring (match-beginning 1) (match-end 1))
+	      tmp-val (mew-buffer-substring (match-beginning 2) (match-end 2))
 	      val (if (string= tmp-val "") val tmp-val)
 	      ret (append ret (mapcar (function (lambda (arg)
 						  (cons (downcase arg) val)))
-				      (mew-fib-split item)))))
-      ret)))
+				      (mew-fib-split item))))))
+    (mew-remove-buffer buf)
+    ret))
 
 (defun mew-fib-fill-default ()
-  "Fill |>item<| acording to the information from .mew-fib"
+  "Fill |>item<| according to the information from .mew-fib"
   (interactive)
   (save-excursion
     (let (begin end str)
@@ -53,7 +53,7 @@
       (while (re-search-forward "|>\\([^<]+\\)<|" nil t)
 	(setq begin (match-beginning 1)
 	      end (match-end 1)
-	      str (buffer-substring begin end))
+	      str (mew-buffer-substring begin end))
 	(delete-region begin end)
 	(backward-char 2)
 	(insert (or (cdr (assoc (downcase str) mew-fib-item-alist)) str)))
@@ -65,8 +65,7 @@
   (save-excursion
     (goto-char (point-min))
     (while (re-search-forward "|>\\|<|" nil t)
-      (replace-match "" nil t)
-      )))
+      (replace-match "" nil t))))
 
 (defun mew-fib-flush-input ()
   "Flush the fib item on the current point."
@@ -100,7 +99,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1994-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
