@@ -2,12 +2,10 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 31, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-header-version "mew-header.el version 0.15")
-
 (require 'mew)
 
 (defvar mew-anonymous-recipients ":;")
@@ -17,23 +15,20 @@
 ;;; Header functions
 ;;;
 
-(defmacro mew-header-goto-next ()
-  '(while (looking-at mew-lwsp)
-     (forward-line)))
+(defsubst mew-header-goto-next ()
+  (while (looking-at mew-lwsp) (forward-line)))
 
 ;; see also mew-header-end
-(defmacro mew-header-goto-end ()
-  '(progn
-     (goto-char (point-min))
-     (if (re-search-forward mew-eoh nil t)
-	 (beginning-of-line)
-       (goto-char (point-max))
-       (if (eolp) (insert "\n")))))
+(defsubst mew-header-goto-end ()
+  (goto-char (point-min))
+  (if (re-search-forward mew-eoh nil t)
+      (beginning-of-line)
+    (goto-char (point-max))
+    (if (eolp) (insert "\n"))))
 
-(defmacro mew-header-goto-body ()
-  '(progn
-     (mew-header-goto-end)
-     (forward-line)))
+(defsubst mew-header-goto-body ()
+  (mew-header-goto-end)
+  (forward-line))
 
 (defun mew-header-get-value (field)
   "currently, when no match, it returns nil."
@@ -53,22 +48,54 @@
 	    (forward-line)
 	    (mew-header-goto-next)
 	    (setq match (mew-buffer-substring start (1- (point))))
-	    (if (string= "" match)
-		()
+	    (unless (string= "" match)
 	      (if ret
 		  (setq ret (concat ret "," match))
 		(setq ret match)
-		(if (equal field mew-from:) (throw 'only-one nil))))))))
+		(if (string= field mew-from:) (throw 'only-one nil))))))))
     ret))
 
-(fset 'mew-header-existp (symbol-function 'mew-header-get-value))
+(defalias 'mew-header-existp 'mew-header-get-value)
+
+(defun mew-header-replace-value (field value)
+  (let ((case-fold-search t)
+	(regex (format "^%s[ \t]*" field))
+	start)
+    (save-excursion
+      (mew-header-goto-end)
+      (save-restriction
+	(narrow-to-region (point-min) (point))
+	(goto-char (point-min))
+	;; make value with evaluating
+	(if (and (listp value)
+		 (stringp (car value)))
+	    (setq value (cons 'format value)))
+	(setq value 
+	      (condition-case e
+		  (eval value)
+		(error 
+		 (format "*** ERROR: %s: %s ***" (car e) (car (cdr e))))))
+	(unless (or (stringp value)
+		    (null value))
+	  (setq value (format "%s" value)))
+	;; at this time, value is string or nil
+	(cond
+	 ((re-search-forward regex nil t)
+	  (setq start (match-beginning 0))
+	  (forward-line)
+	  (mew-header-goto-next)
+	  (delete-region start (point))
+	  (if value
+	      (mew-draft-header-insert field value)))
+	 (value ;; insert when value is exist
+	  (mew-header-goto-end)
+	  (mew-draft-header-insert field value)))))))
 
 (defun mew-make-field-regex (fields)
   (concat "^\\(" (mapconcat (function identity) fields "\\|") "\\)"))
 
 (defun mew-header-delete-lines (fields)
-  (if (null fields)
-      ()
+  (when fields
     (let ((case-fold-search t)
 	  (regex (mew-make-field-regex fields))
 	  start)
@@ -83,8 +110,7 @@
 	  (delete-region start (point)))))))
 
 (defun mew-header-replace-lines (fields prefix)
-  (if (null fields)
-      ()
+  (when fields
     (let ((case-fold-search t)
 	  (regex (mew-make-field-regex fields)))
       (mew-header-goto-end)
@@ -96,23 +122,58 @@
 	  (insert prefix)
 	  (forward-line))))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Header sort
+;;;
+
+(defun mew-header-sort (order)
+  (when order
+    (let* ((case-fold-search t)
+	   (len (length order))
+	   (vec (make-vector len nil))
+	   (i 0)
+	   key beg n line other)
+      (mew-header-goto-end)
+      (save-restriction
+	(narrow-to-region (point-min) (point))
+	(goto-char (point-min))
+	(while (not (eobp))
+	  (if (not (looking-at mew-keyval))
+	      (forward-line)
+	    (setq key (mew-match 1))
+	    (setq beg (match-beginning 0))
+	    (forward-line)
+	    (mew-header-goto-next)
+	    (setq line (mew-buffer-substring beg (point)))
+	    (delete-region beg (point))
+	    (setq n (mew-member-case-equal key order))
+	    (if n
+		(aset vec n (concat (aref vec n) line))
+	      (setq other (concat other line)))))
+	(while (< i len)
+	  (and (aref vec i) (insert (aref vec i)))
+	  (setq i (1+ i)))
+	(and other (insert other))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Header insertion
 ;;;
 
-(defun mew-draft-header-insert (field value)
+(defsubst mew-draft-header-insert (field value)
   "No encoding. Just insert."
-  (if (and value (stringp field))
-      (insert field " " value "\n")))
+  (if (and value (stringp field)) (insert field " " value "\n")))
 
-(defmacro mew-draft-header-fill (field value)
-  (` (if (not (mew-header-existp (, field)))
-	 (mew-draft-header-insert (, field) (, value)))))
+(defsubst mew-draft-header-fill (field value)
+  (unless (mew-header-existp field) (mew-draft-header-insert field value)))
 
-(defun mew-header-insert (key value &optional params)
+(defun mew-header-insert (key value)
   (if (and value (stringp key))
-      (let ((beg (point)) med par parname parval)
+      (let ((beg (point)) params med par parname parval)
+	(when (listp value)
+	  (setq params (cdr value))
+	  (setq value (car value)))
 	(insert key)
   	(insert " ")
 	(setq med (point))
@@ -128,7 +189,7 @@
 	  (cond
 	   ((string-match "^[-a-zA-Z0-9]+$" parval)
 	    ) ;; do nothing
-	   ((and (equal (mew-charset-guess-string parval) mew-us-ascii)
+	   ((and (string= (mew-charset-guess-string parval) mew-us-ascii)
 		 (not (string-match "\"" parval)))
 	    (setq parval (concat "\"" parval "\"")))
 	   (t
@@ -176,18 +237,17 @@
 ;; 'route-addr' will be extracted. So, 'encoded-word' in 'phrase' is
 ;; meaningless.
 
-(defmacro mew-addrstr-parse-syntax-list-check-depth ()
-  '(progn
+(defmacro mew-addrstr-parse-syntax-list-check-depth (&optional depth)
+  `(progn
      (setq sep-cnt (1+ sep-cnt))
-     (if (and (integerp mew-header-max-depth)
-	      (>= sep-cnt mew-header-max-depth))
-	 (progn
-	   (message "Too many values. Truncate values over mew-header-max-depth.")
-	   (ding)
-	   (sit-for 2)
-	   (throw 'max nil)))))
+     (if (and ,depth (>= sep-cnt ,depth))
+	 (throw 'max nil))
+     (when (and (integerp mew-header-max-depth)
+		(>= sep-cnt mew-header-max-depth))
+       (mew-warn "Too many values. Truncate values over mew-header-max-depth.")
+       (throw 'max nil))))
 
-(defun mew-addrstr-parse-syntax-list (str sep addrp)
+(defun mew-addrstr-parse-syntax-list (str sep addrp &optional depth)
   (let* ((i 0) (len (length str))
 	 (par-cnt 0) (tmp-cnt 0) (sep-cnt 0)
 	 (tmp (make-string len ?x))
@@ -239,7 +299,7 @@
 		(setq par-cnt (1+ par-cnt)))
 	       ((char-equal c ?\))
 		(setq par-cnt (1- par-cnt))
-		(if (equal par-cnt 0) (throw 'comment nil))))
+		(if (eq par-cnt 0) (throw 'comment nil))))
 	      (setq i (1+ i)))))
 	 ((char-equal c ?<)
 	  (cond
@@ -249,15 +309,16 @@
 	      (setq rbeg i)
 	      (while (and (< i len) (not (char-equal (aref str i) ?>)))
 		(setq i (1+ i)))
-	      (setq rend i);; note: to be used for substring, so not 1-.
-	      ;; should not be nested but easy to implement...
-	      (setq ret (cons (car (mew-addrstr-parse-syntax-list
-				    (substring str rbeg rend) sep t))
-			      ret)))
+	      (when (> i rbeg)
+		(setq rend i) ;; note: to be used for substring, so not 1-.
+		;; should not be nested but easy to implement...
+		(setq ret (cons (car (mew-addrstr-parse-syntax-list
+				      (substring str rbeg rend) sep t))
+				ret))))
 	    (while (and (< i len) (not (char-equal (aref str i) sep)))
 	      (setq i (1+ i)))
 	    (setq tmp-cnt 0)
-	    (mew-addrstr-parse-syntax-list-check-depth))
+	    (mew-addrstr-parse-syntax-list-check-depth depth))
 	   (t
 	    ;; just ignore
 	    (while (and (< i len) (not (char-equal (aref str i) ?>)))
@@ -269,14 +330,13 @@
 	  (if (> tmp-cnt 0)
 	      (setq ret (cons (substring tmp 0 tmp-cnt) ret)))
 	  (setq tmp-cnt 0)
-	  (mew-addrstr-parse-syntax-list-check-depth))
+	  (mew-addrstr-parse-syntax-list-check-depth depth))
 	 (t 
 	  (aset tmp tmp-cnt c)
-	  (setq tmp-cnt (1+ tmp-cnt)))
-	 )
-	(setq i (1+ i))))
-    (if (> tmp-cnt 0)
-	(setq ret (cons (substring tmp 0 tmp-cnt) ret)))
+	  (setq tmp-cnt (1+ tmp-cnt)))) ;; end of cond
+	(setq i (1+ i))) ;; end of while
+      (if (> tmp-cnt 0)
+	  (setq ret (cons (substring tmp 0 tmp-cnt) ret)))) ;; end of catch
     (nreverse ret)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -284,33 +344,36 @@
 ;;; High-level functions to parse fields
 ;;;
 
-(defmacro mew-addrstr-parse-value (value)
-  (` (car (mew-addrstr-parse-value-list (, value)))))
+(defsubst mew-addrstr-parse-value (value)
+  (car (mew-addrstr-parse-syntax-list value ?\; nil 1)))
 
-(defmacro mew-addrstr-parse-value-list (value)
-  (` (mew-addrstr-parse-syntax-list (, value) ?\; nil)))
+(defsubst mew-addrstr-parse-value-list (value)
+  (mew-addrstr-parse-syntax-list value ?\; nil))
 
-(defmacro mew-addrstr-parse-value2 (value)
-  (` (car (mew-addrstr-parse-value-list2 (, value)))))
+(defsubst mew-addrstr-parse-value2 (value)
+  (car (mew-addrstr-parse-syntax-list value ?, nil 1)))
 
-(defmacro mew-addrstr-parse-value-list2 (value)
-  (` (mew-addrstr-parse-syntax-list (, value) ?, nil)))
+(defsubst mew-addrstr-parse-value-list2 (value)
+  (mew-addrstr-parse-syntax-list value ?, nil))
 
 ;;
 
-(defmacro mew-addrstr-parse-address (address)
-  (` (car (mew-addrstr-parse-address-list (, address)))))
+(defsubst mew-addrstr-parse-address (address)
+  (car (mew-addrstr-parse-syntax-list address ?, t 1)))
 
-(defmacro mew-addrstr-parse-address-list (address)
-  (` (mew-addrstr-parse-syntax-list (, address) ?, t)))
+(defsubst mew-addrstr-parse-address-list (address)
+  (mew-addrstr-parse-syntax-list address ?, t))
 
-(defmacro mew-header-parse-address (field)
-  (` (car (mew-header-parse-address-list (list (, field))))))
+(defsubst mew-header-parse-address (field)
+  (car
+   (mew-addrstr-parse-syntax-list
+    (mapconcat (function mew-header-get-value) (list field) ",")
+    ?, t 1)))
 
-(defmacro mew-header-parse-address-list (field-list)
-  (` (mew-addrstr-parse-syntax-list
-      (mapconcat (function mew-header-get-value) (, field-list) ",")
-      ?, t)))
+(defsubst mew-header-parse-address-list (field-list)
+  (mew-addrstr-parse-syntax-list
+   (mapconcat (function mew-header-get-value) field-list ",")
+   ?, t))
 
 (defun mew-header-parse-address-list2 (field-list)
   "Collect addresses from FIELD-LIST. 
@@ -320,15 +383,15 @@ Remove anonymous addresses."
 	       ?, t))
 	ret)
     (while vals
-      (if (not (string-match mew-anonymous-recipients (car vals)))
-	  (setq ret (cons (car vals) ret)))
+      (unless (string-match mew-anonymous-recipients (car vals))
+	(setq ret (cons (car vals) ret)))
       (setq vals (cdr vals)))
     (nreverse ret)))
 
 ;;
 
-(defmacro mew-addrstr-extract-user-list (addr-list)
-  (` (mapcar (function mew-addrstr-extract-user) (, addr-list))))
+(defsubst mew-addrstr-extract-user-list (addr-list)
+  (mapcar (function mew-addrstr-extract-user) addr-list))
 
 (defun mew-addrstr-extract-user (addr)
   "Extracts username from ADDR"
@@ -348,21 +411,18 @@ Remove anonymous addresses."
   (if mew-addrbook-append-domain-p
       (if (string-match "@" addr)
 	  addr
-	(concat addr "@" mew-mail-domain))
+	(concat addr "@" (mew-mail-domain))) ;; xxx
     addr))
 
 (defun mew-addrstr-expand-alias (alias)
-  (if (and mew-addrbook-unexpand-regex
-	   (string-match mew-addrbook-unexpand-regex alias))
-      (list alias) ;; not expand
-    (let ((addrs (mew-alias-get alias)))
-      (setq addrs (mapcar (function mew-chop) (mew-split addrs ?,)))
-      (setq addrs (delete "" addrs))
-      (mapcar (function mew-addrstr-append-domain) addrs))))
+  (let ((addrs (mew-alias-get alias)))
+    (setq addrs (mapcar (function mew-chop) (mew-split addrs ?,)))
+    (setq addrs (delete "" addrs))
+    (mapcar (function mew-addrstr-append-domain) addrs)))
 
 (defun mew-addrstr-canonicalize-address (addr-str)
   "Expand aliases for each addresses in ADDR-STR. 
-Then prepend mew-mail-domain if a domain part doesn't exist."
+Then prepend (mew-mail-domain) if a domain part doesn't exist."
   (let ((addrs (mapcar (function mew-chop) (mew-split addr-str ?,)))
 	addr ret)
     (while addrs
@@ -375,7 +435,7 @@ Then prepend mew-mail-domain if a domain part doesn't exist."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1996-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
