@@ -2,90 +2,56 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Oct  2, 1996
-;; Revised: Aug 24, 1998
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-message-version "mew-message.el version 0.11")
-
 (require 'mew)
-(if mew-xemacs-p (require 'easymenu))
 
-(defvar mew-message-mode-map nil)
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Message info
+;;;
 
-(defvar mew-message-mode-menu-spec
-  '("Mew/message"
-    ["Next part"    mew-message-next-msg t]
-    ["Prev part"    mew-message-prev-msg t]
-    ["Next page"    mew-message-next-page t]
-    ["Prev page"    mew-message-prev-page t]
-    ["Goto summary" mew-message-goto-summary t]
-    "---"
-    ("Reply/Forward"
-     ["Reply"               mew-message-reply t]
-     ["Reply with citation" mew-message-reply-with-citation t]
-     ["Forward"             mew-message-forward t]
-     )
-    )
-  )
+(defvar mew-minfo-list '("summary" "overlay"))
 
-(if mew-message-mode-map
-    ()
-  (setq mew-message-mode-map (make-sparse-keymap))
-  (define-key mew-message-mode-map " "    'mew-message-next-page)
-  (define-key mew-message-mode-map "\177" 'mew-message-prev-page)
-  (define-key mew-message-mode-map "n"    'mew-message-next-msg)
-  (define-key mew-message-mode-map "p"    'mew-message-prev-msg)
-  (define-key mew-message-mode-map "h"    'mew-message-goto-summary)
-  (define-key mew-message-mode-map "a"    'mew-message-reply)
-  (define-key mew-message-mode-map "A"    'mew-message-reply-with-citation)
-  (define-key mew-message-mode-map "f"    'mew-message-forward)
-  (define-key mew-message-mode-map "r"    'mew-message-resend)
-  (if mew-temacs-p
-      (easy-menu-define
-       mew-message-mode-menu
-       mew-message-mode-map
-       "Menu used in Message mode."
-       mew-message-mode-menu-spec)
-    )
-  )
+(mew-blinfo-defun 'mew-minfo mew-minfo-list)
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Message mode
 ;;;
 
 (defun mew-message-mode ()
-  "Major mode for display a message.
+  "\\<mew-message-mode-map>
+Mew Message mode:: major mode for display a message.
 The keys that are defined for this mode are:
 
-SPC	Scroll up this message.
-DEL	Back-scroll this message.
-n	Display a message or a part below.
-p	Display a message or a part above.
-h	Get back to Summary mode.
-a	Answer to this message. A new draft is prepared in Draft mode. 
+\\[mew-message-next-page]	Scroll up this message.
+\\[mew-message-prev-page]	Back-scroll this message.
+\\[mew-message-next-msg]	Display a message or a part below.
+\\[mew-message-prev-msg]	Display a message or a part above.
+\\[mew-message-goto-summary]	Get back to Summary mode.
+\\[mew-message-reply]	Answer to this message. A new draft is prepared in Draft mode. 
 	Mew automatically decides To: and Cc:.
-A	Answer to this message. A new draft is prepared in Draft mode. 
+\\[mew-message-reply-with-citation]	Answer to this message. A new draft is prepared in Draft mode. 
 	Mew automatically decides To: and Cc: and cites the body.
-f	Forward this message to a third person. A new draft is prepared in 
+\\[mew-message-forward]	Forward this message to a third person. A new draft is prepared in 
 	Draft mode and this message is automatically attached.
-r	Resend this message with Resent-To:. It is strongly 
-	discouraged to use this command since beginners are always 
-	confused. Please use 'f' instead.
 "
   (interactive)
   (setq major-mode 'mew-message-mode)
-  (setq mode-name "Message")
+  (setq mode-name mew-mode-name-message)
   (setq mode-line-buffer-identification mew-mode-line-id)
   (use-local-map mew-message-mode-map)
   (setq buffer-read-only t)
   (make-local-variable 'page-delimiter)
   (setq page-delimiter mew-page-delimiter)
-  (run-hooks 'mew-message-mode-hook)
-  )
+  (mew-message-toolbar-update)
+  (run-hooks 'mew-message-mode-hook))
 
 (defun mew-message-next-page (&optional lines)
-  "Scroll up this message. Return nil if more pages. Otherwise, return t."
+  "Scroll up this message. Return 'nil' if more pages. Otherwise, return 't'."
   (interactive)
   (move-to-window-line -1)
   (if (save-excursion
@@ -99,19 +65,18 @@ r	Resend this message with Resent-To:. It is strongly
 	  t
 	;; Go to the next page.
 	(mew-message-narrow-to-page 1)
-	nil
-	)
+	nil)
     ;; More in this page.
     (condition-case nil
 	(scroll-up lines)
       (end-of-buffer
        (goto-char (point-max))
        (message "End of buffer")))
-    nil
-    ))
+    nil))
 
 (defun mew-message-prev-page (&optional lines)
-  "Back-scroll this message. Return nil if more pages. Otherwise, return t."
+  "Back-scroll this message. Return 'nil' if more pages. 
+Otherwise, return 't'."
   (interactive)
   (move-to-window-line 0)
   (if (save-excursion
@@ -124,16 +89,14 @@ r	Resend this message with Resent-To:. It is strongly
 	  t
 	;; Go to the previous page.
 	(mew-message-narrow-to-page -1)
-	nil
-	)
+	nil)
     ;; More in this page.
     (condition-case nil
 	(scroll-down lines)
       (beginning-of-buffer
        (goto-char (point-min))
        (message "Beginning of buffer")))
-    nil
-    ))
+    nil))
 
 (defun mew-message-narrow-to-page (&optional arg)
   (interactive "P")
@@ -142,31 +105,36 @@ r	Resend this message with Resent-To:. It is strongly
     (condition-case nil
 	(forward-page -1) ;; Beginning of the current page.
       (beginning-of-buffer ()))
-    (forward-char 1)  ;; for compatibility with emacs-19.28 and emacs-19.29
     (widen)
     (cond
      ((> arg 0)	(forward-page arg))
      ((< arg 0) (forward-page (1- arg))))
     (forward-page)
-    (narrow-to-region
-     (point)
-     (progn
-       (forward-page -1)
-       (if (and (eolp) (not (bobp)))
-	   (forward-line))
-       (point)))))
+    (let* ((end (point)))
+      (forward-page -1)
+      (if (and (eolp) (not (bobp)))
+	  (forward-line))
+      (narrow-to-region (point) end))))
 
 (defun mew-message-goto-summary ()
   "Get back to Summary mode."
   (interactive)
-  (let* ((sum-num (mew-current-get 'message))
-	 (sum (car sum-num))
-	 (num (cdr sum-num)))
-    (if (not (get-buffer sum))
-	(message "No Summary mode for %s" sum)
-      (mew-pop-to-buffer sum)
-      (if num
-	  (mew-summary-jump-message num)))))
+  (let* ((fld (mew-minfo-get-summary))
+	 (msg (mew-current-get-msg (mew-frame-id)))
+	 (part (mew-current-get-part (mew-frame-id))))
+    (if (not (get-buffer fld))
+	(message "No Summary mode for %s" fld)
+      (mew-pop-to-buffer fld)
+      (cond
+       ((and msg part (mew-decode-syntax-p))
+	(setq part (concat "^.. +"
+			   (regexp-quote (mapconcat 'int-to-string part "."))
+			   " "))
+	(goto-char (mew-decode-syntax-begin))
+	(re-search-forward part (mew-decode-syntax-end) t)
+	(beginning-of-line)
+	(mew-summary-display nil))
+       (msg (mew-summary-jump-message msg))))))
 
 (defun mew-message-reply ()
   "Answer to this message. A new draft is prepared in Draft mode. 
@@ -190,9 +158,10 @@ Draft mode and this message is automatically attached."
   (call-interactively 'mew-summary-forward))
 
 (defun mew-message-resend ()
-  "Resend this message with Resent-To:. It is strongly 
+  "\\<mew-message-mode-map>
+Resend this message with Resent-To:. It is strongly 
 discouraged to use this command since beginners are always 
-confused. Please use 'f' instead."
+confused. Please use '\\[mew-message-forward]' instead."
   (interactive)
   (mew-message-goto-summary)
   (call-interactively 'mew-summary-resend))
@@ -201,14 +170,12 @@ confused. Please use 'f' instead."
   "Display a message or a part below."
   (interactive "p")
   (let* ((obuf (current-buffer))
-	 (buf (window-buffer (previous-window)))
-	 (buffer-read-only nil))
+	 (buf (window-buffer (previous-window))))
     (mew-pop-to-buffer buf) ;; for the next forward-line
-    (if (not (or (eq major-mode 'mew-summary-mode)
-		 (eq major-mode 'mew-virtual-mode)))
+    (if (not (mew-summary-or-virtual-p))
 	()
       (forward-line arg) ;; minus arg results in prev
-      (mew-summary-display))
+      (mew-summary-display 'redisplay))
     ;; for window config
     (mew-pop-to-buffer obuf)))
       
@@ -221,7 +188,7 @@ confused. Please use 'f' instead."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998 Mew developing team.
+;; Copyright (C) 1996-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
