@@ -2,11 +2,16 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 20, 1997
-;; Revised: Aug 30, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-mule0-version "mew-mule0.el version 0.10")
+;; must be here
+(defun mew-coding-system-p (cs)
+  (cond
+   ((null cs) t)
+   ((eq cs 'iso-8859-1) t)
+   (t nil)))
 
 ;; In the context of Mew, 'charset' means MIME charset.
 ;; 'cs' means the internal representation of Emacs (was known as Mule).
@@ -19,102 +24,93 @@
 (defvar mew-cs-binary         nil)
 (defvar mew-cs-text-for-read  nil)
 (defvar mew-cs-text-for-write nil)
+(defvar mew-cs-text-for-net   nil)
 (defvar mew-cs-autoconv       nil)
-(defvar mew-cs-7bit           nil)
-(defvar mew-cs-7bit-crlf      nil)
+(defvar mew-cs-m17n           nil)
 
-(defvar mew-cs-mime-trans     nil)
-(defvar mew-cs-rfc822-trans   nil)
-(defvar mew-cs-draft          nil)
-(defvar mew-cs-scan           nil)
-(defvar mew-cs-infile         nil)
-(defvar mew-cs-outfile        nil)
-(defvar mew-cs-virtual        nil)
-(defvar mew-cs-pick           nil)
+(defvar mew-cs-eol "\r\n")
 
-(defvar mew-cs-database
-  '(("us-ascii"      (0)   nil   "7bit"             nil "Q")
-    ("iso-8859-1"    (0 1) dummy "quoted-printable" nil "Q")))
+(defvar mew-cs-database-for-encoding
+  '(((ascii)                 nil        "7bit"             "7bit")
+    ((ascii latin-iso8859-1) iso-8859-1 "quoted-printable" "Q")))
 
-;;
-;; Leading characters
-;;
-
-(defvar mew-lc-ascii 0)
-(defvar mew-lc-kana  nil) ;; dummay
-
-(defun mew-char-charset (char)
-  ())
+(defvar mew-cs-database-for-decoding
+  '(("us-ascii"   . nil)
+    ("iso-8859-1" . iso-8859-1)))
 
 ;;
 ;; CS
 ;;
 
-(defun mew-find-cs-region (start end)
+(defun mew-find-cs-region (beg end)
   (let (ret)
     (save-excursion
       (goto-char beg)
       (if (re-search-forward "[\000-\177]" end t)
-	  (setq ret (cons 0 ret)))
+	  (setq ret (cons 'ascii ret)))
       (goto-char beg)
       (if (re-search-forward "[\200-\377]" end t)
-	  (setq ret (cons 1 ret))))
+	  (setq ret (cons 'latin-iso8859-1 ret))))
     ret))
 
 ;; to internal
-(defmacro mew-cs-decode-region (beg end cs)
-  (` ()))
+(defsubst mew-cs-decode-region (beg end cs) nil)
 
 ;; to extenal
-(defmacro mew-cs-encode-region (beg end cs)
-  (` ()))
+(defsubst mew-cs-encode-region (beg end cs) nil)
 
 ;; to internal
-(defmacro mew-cs-decode-string (str cs)
-  (` (, str)))
+(defsubst mew-cs-decode-string (str cs) str)
 
 ;; to external
-(defmacro mew-cs-encode-string (str cs)
-  (` (, str)))
+(defsubst mew-cs-encode-string (str cs) str)
 
 ;;
 ;; Process environment
 ;;
 
-(defmacro mew-set-process-cs (pro from-pro to-pro)
-  (` ()))
+(defsubst mew-set-process-cs (pro from-pro to-pro) nil)
+
+(defsubst mew-set-buffer-cs (write) nil)
 
 (defmacro mew-plet (&rest body)
-  (` (let ((call-process-hook nil))
-       (,@ body))))
+  `(progn ,@body))
 
 (defmacro mew-piolet (input output &rest body)
-  (` (let ((call-process-hook nil))
-       (,@ body))))
+  `(progn ,@body))
+
+(defmacro mew-pioalet (input output arg &rest body)
+  `(progn ,@body))
 
 (defmacro mew-flet (&rest body)
-  (` (let (jam-zcat-filename-list
-	   jka-compr-compression-info-list)
-       (,@ body))))
+  `(let ((jka-compr-inhibit t))
+     ,@body))
 
 (defmacro mew-frwlet (read write &rest body)
-  (` (let (jam-zcat-filename-list
-	   jka-compr-compression-info-list)
-       (,@ body))))
-
-;;
-;; Post conversion
-;;
-
-(defmacro mew-cs-post-conv (cs)
-  (` ()))
+  `(let ((jka-compr-inhibit t))
+     ,@body))
 
 ;;
 ;;
 ;;
 
-(fset 'mew-aref (symbol-function 'aref))
-(defmacro mew-charlen (c) (` 1))
+(defun mew-substring (str width &optional cnt)
+  (let ((sw (length str)))
+    (cond
+     ((= sw width)
+      str)
+     ((< sw width)
+      (concat str (make-string (- width sw) 32)))
+     (t
+      (if cnt
+	  (concat (substring str 0 (- width 2)) "..")
+	(substring str 0 width))))))
+
+;;
+;; Language specific
+;;
+
+(defvar mew-lc-kana  nil) ;; dummy
 
 ;;
 ;;
@@ -125,7 +121,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1997-2000 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
