@@ -3,12 +3,10 @@
 ;; Author:  Yoshinari NOMURA <nom@csce.kyushu-u.ac.jp>
 ;;          Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Jun 11, 1994
-;; Revised: Aug 30, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-refile-version "mew-refile.el version 0.76")
-
 (require 'mew)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -18,76 +16,37 @@
 
 (defvar mew-refile-msgid-alist nil
   "Alist of message-id and folder pair")
-(defvar mew-refile-msgid-file-name ".mew-refile-msgid-alist")
+
 (defvar mew-refile-from-alist nil
   "Alist of From: address and folder pair")
-(defvar mew-refile-from-file-name ".mew-refile-from-alist")
 
 (defvar mew-refile-last-folder nil
   "Folder name previously you refiled")
 
-(defvar mew-refile-ctrl-multi t
-  "*If *non-nil*, guess functions guess multi folders.")
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; initialize function
+;;;
 
-(defvar mew-refile-guess-alist nil
-  "*If non-nil, mew guesses destination folder by using this hint.
-The format is like this:
-
-    (setq mew-refile-guess-alist
-          '((\"To:\" 
-              (\"wide@wide\" . \"+wide/wide\")
-              (\"adam\"      . \"+labo/adam\"))
-            (\"Newsgroups:\"
-              (\"^nifty\\\\.\\\\([^ ]+\\\\)\" . \"+Nifty/\\\\1\"))
-            (\"From:\" 
-              (\"uucp\" . \"+adm/uucp\")
-              (\".*\"   . \"+misc\"))
-            ))
-")
-
-(defvar mew-refile-guess-key-list mew-destination:-list
-  "*A list of field key used by mew-refile-guess-by-folder.")
-
-(defvar mew-refile-guess-control
-  '(mew-refile-guess-by-alist
-    mew-refile-ctrl-throw
-    mew-refile-guess-by-newsgroups
-    mew-refile-guess-by-folder
-    mew-refile-ctrl-throw
-    mew-refile-ctrl-auto-boundary
-    mew-refile-guess-by-thread
-    mew-refile-ctrl-throw
-    mew-refile-guess-by-from-folder
-    mew-refile-ctrl-throw
-    mew-refile-guess-by-from
-    mew-refile-ctrl-throw
-    mew-refile-guess-by-default))
-
-(defvar mew-refile-auto-refile-skip-any-mark nil
-  "*If *non-nil*, 'mew-summary-auto-refile' doesn't touch
-any alredy marked message.")
-
-(defvar mew-refile-auto-refile-confirm nil
-  "*If *non-nil*, 'mew-summary-auto-refile' prompts the user for
-confirmation before refiling.")
-
-(defvar mew-refile-guess-strip-domainpart t
-  "*If *non-nil*, 'mew-refile-guess-by-default' strips domainpart of from")
-
-(defvar mew-refile-guess-from-me-is-special nil
-  "*If *non-nil*, 'mew-refile-guess-by-from-*' think the mails from yourself
-as special. They use To: or Cc: instead of From:")
-
-;;
-;; initialize function
-;;
-(defun mew-refile-init ()
+(defun mew-refile-setup ()
   ;; load message id alist
   (or mew-refile-msgid-alist
-      (setq mew-refile-msgid-alist (mew-lisp-load mew-refile-msgid-file-name)))
+      (setq mew-refile-msgid-alist (mew-lisp-load mew-refile-msgid-file)))
   ;; load from alist
   (or mew-refile-from-alist
-      (setq mew-refile-from-alist (mew-lisp-load mew-refile-from-file-name))))
+      (setq mew-refile-from-alist (mew-lisp-load mew-refile-from-file)))
+  (add-hook 'kill-emacs-hook (function mew-refile-clean-up)))
+
+(defun mew-refile-clean-up ()
+  (remove-hook 'kill-emacs-hook (function mew-refile-clean-up))
+  (if (and mew-refile-from-alist
+	   (member 'mew-refile-guess-by-from mew-refile-guess-control))
+      (mew-lisp-save mew-refile-from-file mew-refile-from-alist))
+  (if (and mew-refile-msgid-alist
+	   (member 'mew-refile-guess-by-thread mew-refile-guess-control))
+      (mew-lisp-save mew-refile-msgid-file mew-refile-msgid-alist))
+  (setq mew-refile-from-alist nil)
+  (setq mew-refile-msgid-alist nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -107,6 +66,7 @@ as special. They use To: or Cc: instead of From:")
 ;; that is, 'car' is a list of judged  folders.
 ;;          'cdr' is a alist of opinions by guess functions.
 ;;
+
 (defun mew-refile-guess (&optional auto show-all)
   (let ((funcs mew-refile-guess-control) ret guess info stop)
     (catch 'last
@@ -114,10 +74,9 @@ as special. They use To: or Cc: instead of From:")
 	(cond
 	 ;; func is control function
 	 ((string-match "ctrl" (symbol-name (car funcs)))
-	  (if (setq ret (funcall (car funcs) guess auto))
-	      (progn
-		(setq stop t)
-		(or show-all (throw 'last t)))))
+	  (when (setq ret (funcall (car funcs) guess auto))
+	    (setq stop t)
+	    (or show-all (throw 'last t))))
 	 ;; func is guess function
 	 (t
 	  (setq ret (funcall (car funcs)))))
@@ -127,7 +86,7 @@ as special. They use To: or Cc: instead of From:")
 	;; shape the guess: (guess1 guess2 ...)
 	;;            info: (info1 info2 ...)
 	(setq info (nconc info (list (cons (car funcs) ret))))
-	(if (not stop) (setq guess (nconc (reverse ret) guess)))
+	(unless stop (setq guess (nconc (reverse ret) guess)))
 	(setq funcs (cdr funcs))))
     (setq guess (nreverse guess))
     (if mew-refile-ctrl-multi
@@ -138,15 +97,17 @@ as special. They use To: or Cc: instead of From:")
 ;;
 ;; guess control functions
 ;;
+
 (defun mew-refile-ctrl-auto-boundary (guess auto)
-  (if auto 'stop))
+  (if auto "stop"))
 
 (defun mew-refile-ctrl-throw (guess auto)
-  (if guess 'stop))
+  (if guess "stop"))
 
 ;;
 ;; by alist returns: (guess1 guess2 ...) or nil
 ;;
+
 (defun mew-refile-guess-by-alist ()
   (mew-refile-guess-by-alist1 mew-refile-guess-alist))
 
@@ -162,21 +123,22 @@ as special. They use To: or Cc: instead of From:")
 	(or ret (setq ret (cons sublist ret))))
        (t
 	(setq header (mew-header-get-value name))
-	(if header
-	    (while sublist
-	      (setq key (car (car sublist)))
-	      (setq val (cdr (car sublist)))
-	      (if (and (stringp key) (string-match key header))
-		  (cond
-		   ((stringp val)
-		    (setq ent (mew-refile-guess-by-alist2 key header val)))
-		   ((listp val)
-		    (setq ent (mew-refile-guess-by-alist1 val)))))
-	      (if ent
-		  (if (listp ent)
-		      (setq ret (nconc ent ret) ent nil)
-		    (setq ret (cons ent ret))))
-	      (setq sublist (cdr sublist))))))
+	(when header
+	  (while sublist
+	    (setq key (car (car sublist)))
+	    (setq val (cdr (car sublist)))
+	    (setq ent nil)
+	    (when (and (stringp key) (string-match key header))
+	      (cond
+	       ((stringp val)
+		(setq ent (mew-refile-guess-by-alist2 key header val)))
+	       ((listp val)
+		(setq ent (mew-refile-guess-by-alist1 val)))))
+	    (when ent
+	      (if (listp ent)
+		  (setq ret (nconc ent ret) ent nil)
+		(setq ret (cons ent ret))))
+	    (setq sublist (cdr sublist))))))
       (setq alist (cdr alist)))
     (mew-uniq-list (nreverse ret))))
 
@@ -190,14 +152,12 @@ as special. They use To: or Cc: instead of From:")
 			     (car match-list) (car (cdr match-list)))
 		  match-strings))
       (setq match-list (cdr (cdr match-list))))
+    ;; xxx should use replace-match
     (while (string-match "\\\\\\([1-9]\\)" string)
 	(setq string
 	      (concat (substring string 0 (match-beginning 0))
 		      (nth (- (length match-strings)
-			      (string-to-int 
-			       (substring string
-					  (match-beginning 1)
-					  (match-end 1))))
+			      (string-to-int (mew-match 1 string)))
 			   match-strings)
 		      (substring string (match-end 0)))))
     string))
@@ -205,12 +165,12 @@ as special. They use To: or Cc: instead of From:")
 ;;
 ;; by newsgroups returns (guess1 guess2 ...) or nil
 ;;
+
 (defun mew-refile-guess-by-newsgroups ()
   (let ((newsgroups (mew-addrstr-parse-value-list2 
 		     (mew-header-get-value mew-newsgroups:)))
 	ent ret)
-    (if (not newsgroups)
-	()
+    (when newsgroups
       (while newsgroups
 	(setq ent (mew-assoc-case-equal (car newsgroups) mew-folder-alist 1))
 	(if ent (setq ret (cons (nth 0 ent) ret)))
@@ -220,33 +180,36 @@ as special. They use To: or Cc: instead of From:")
 ;;
 ;; by folder returns: (guess1 guess2 ...) or nil
 ;;
+
 (defun mew-refile-guess-by-folder ()
   (let ((to-cc (mew-header-parse-address-list mew-refile-guess-key-list))
-	ent ret ml-name)
+	ent ret ml-addr ml-name)
     (while to-cc
-      (setq ml-name (mew-addrstr-extract-user (or (car to-cc) "")))
-      (setq ent (mew-assoc-case-equal ml-name mew-folder-alist 1))
-      (if ent (setq ret (cons (nth 0 ent) ret)))
+      (setq ml-addr (mew-addrstr-parse-address (or (car to-cc) "")))
+      (when ml-addr
+	(setq ml-name (mew-addrstr-extract-user ml-addr))
+	(setq ent (or (mew-assoc-case-equal ml-addr mew-folder-alist 1)
+		      (mew-assoc-case-equal ml-name mew-folder-alist 1)))
+	(if ent (setq ret (cons (nth 0 ent) ret))))
       (setq to-cc (cdr to-cc)))
     (mew-uniq-list (nreverse ret))))
 
 ;;
 ;; by message-thread returns: guess1 or nil
 ;;
+
 (defun mew-refile-guess-by-thread ()
   (let ((msgid (or (mew-header-get-value mew-references:)
 		   (mew-header-get-value mew-in-reply-to:))))
     ;; search for msgid
     (if (and msgid 
 	     (string-match "\\(<[^ \t\n]*>\\)[^>]*\0" (concat msgid "\0")))
-	(nth 1 (assoc (substring msgid 
-				 (match-beginning 1)
-				 (match-end 1))
-		      mew-refile-msgid-alist)))))
+	(nth 1 (assoc (mew-match 1 msgid) mew-refile-msgid-alist)))))
 
 ;;
 ;; by from-{folder,alist} returns: guess1 or (guess1 ...) or nil
 ;;
+
 (defun mew-refile-guess-by-from-folder (&optional addr)
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-folder-body addr))
 
@@ -254,10 +217,11 @@ as special. They use To: or Cc: instead of From:")
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-body addr))
 
 (defun mew-refile-guess-by-from-folder-body (&optional addr)
-  (let* ((pfix (cond ((equal mew-folders-default-folder "" ) "+")
-		     ((equal mew-folders-default-folder nil) "+")
-		     ((equal mew-folders-default-folder "+") "+")
-		     (t (file-name-as-directory mew-folders-default-folder))))
+  (let* ((pfix (cond
+		((string= mew-folders-default-folder "" ) "+")
+		((string= mew-folders-default-folder "+") "+")
+		((eq mew-folders-default-folder nil)      "+")
+		(t (file-name-as-directory mew-folders-default-folder))))
 	 (from (downcase (or addr (mew-header-parse-address mew-from:) "")))
 	 (user (mew-addrstr-extract-user from))
 	 (pfix-regex (concat "^" (regexp-quote pfix) "\\(.*/\\)?"))
@@ -275,11 +239,12 @@ as special. They use To: or Cc: instead of From:")
 ;; dispatcher to make mew-refile-guess-by-from-* consider
 ;; mew-refile-guess-from-me-is-special.
 ;;
+
 (defun mew-refile-guess-from-dispatch (func &optional addr)
   (let ((addr (downcase (or addr (mew-header-parse-address mew-from:) ""))))
     ;; if From: is my address, addr is the list extracted from To:, Cc:
     (if (and mew-refile-guess-from-me-is-special
-	     (mew-is-my-address (mew-get-my-address-regex-list) addr))
+	     (mew-is-my-address mew-regex-my-address-list addr))
 	(let ((addr (mew-header-parse-address-list mew-refile-guess-key-list))
 	      (a nil) (r nil))
 	  (while addr
@@ -292,14 +257,16 @@ as special. They use To: or Cc: instead of From:")
 ;;
 ;; by default returns: guess1
 ;;
+
 (defun mew-refile-guess-by-default (&optional addr)
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-default-body addr))
 
 (defun mew-refile-guess-by-default-body (&optional addr)
-  (let* ((pfix (cond ((equal mew-folders-default-folder "" ) "+")
-		     ((equal mew-folders-default-folder nil) "+")
-		     ((equal mew-folders-default-folder "+") "+")
-		     (t (file-name-as-directory mew-folders-default-folder))))
+  (let* ((pfix (cond
+		((string= mew-folders-default-folder "" ) "+")
+		((string= mew-folders-default-folder "+") "+")
+		((eq mew-folders-default-folder nil)      "+")
+		(t (file-name-as-directory mew-folders-default-folder))))
 	 (from (downcase (or addr (mew-header-parse-address mew-from:) "")))
 	 (user (mew-addrstr-extract-user from)))
     (if mew-refile-guess-strip-domainpart
@@ -324,6 +291,7 @@ as special. They use To: or Cc: instead of From:")
 ;; that is, 'car' is a list of user chosen folders.
 ;;          'cdr' is a list of opinions by guess functions.
 ;;
+
 (defun mew-refile-guess-learn (buf result)
   (let ((chosen (car result))  ;; (folder1 folder2 ...)
 	(info   (cdr result))) ;; (guess-func-name guess1 guess2...)
@@ -336,6 +304,7 @@ as special. They use To: or Cc: instead of From:")
 ;;
 ;; learn from msgid
 ;;
+
 (defun mew-refile-guess-by-thread-learn (chosen info)
   (let* ((msgid  (mew-header-get-value mew-message-id:)) 
 	 (folder (car chosen))
@@ -343,9 +312,8 @@ as special. They use To: or Cc: instead of From:")
 	 (oho    info)
 	 (mho    (cdr (assoc 'mew-refile-guess-by-thread info))))
     (if (and msgid (string-match "<[^ \n>]*>" msgid))
-	(setq msgid (substring msgid (match-beginning 0) (match-end 0))))
-    (if (or (not msgid) (not chosen))
-	()
+	(setq msgid (mew-match 0 msgid)))
+    (when (and msgid chosen)
       ;; if my opninion was right, I learn it.
       ;; or a folder was not in other people's opinion,
       ;; I accept it.
@@ -360,11 +328,12 @@ as special. They use To: or Cc: instead of From:")
 	  (setq chosen (cdr chosen))))
       (setq mew-refile-msgid-alist
 	    (cons (list msgid folder "??")
-		  (delete (assoc msgid mew-refile-msgid-alist)
-			  mew-refile-msgid-alist))))))
+		  (delq (assoc msgid mew-refile-msgid-alist) ;; delq is right
+			mew-refile-msgid-alist))))))
 ;;
 ;; learn from "From:" field
 ;;
+
 (defun mew-refile-guess-by-from-learn (chosen info)
   (let* ((from  (downcase (or (mew-header-parse-address mew-from:) "")))
 	 (folder nil)
@@ -374,36 +343,27 @@ as special. They use To: or Cc: instead of From:")
     (if (or (or (null from) (null chosen)) ;; if from and/or chosen is empty
 	    (and mho (member mho chosen))) ;; or my opinion was right,
 	()                                 ;; do nothing.
-
       ;; I decide which folder is most important among the user chosen
       ;; folders. 
       (catch 'match
 	(while chosen
 	  ;; searching a folder anyone couldn't predict.
-	  (if (not (mew-refile-guess-member-p (car chosen) info))
-	      (throw 'match (setq folder (car chosen))))
+	  (unless (mew-refile-guess-member-p (car chosen) info)
+	    (throw 'match (setq folder (car chosen))))
 	  (setq chosen (cdr chosen))))
 
       ;; If candidate was found, I memorize it.
-      (if folder
-	  (setq mew-refile-from-alist
-		(cons (cons from folder)
-		      (delete (assoc from mew-refile-from-alist)
-			      mew-refile-from-alist)))))))
+      (when folder
+	(setq mew-refile-from-alist
+	      (cons (cons from folder)
+		    (delq (assoc from mew-refile-from-alist) ;; delq is right
+			  mew-refile-from-alist)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Backyard functions.
 ;;;
 
-(defun mew-refile-guess-save ()
-  (if (and mew-refile-from-alist
-	   (member 'mew-refile-guess-by-from mew-refile-guess-control))
-      (mew-lisp-save mew-refile-from-file-name mew-refile-from-alist))
-  (if (and mew-refile-msgid-alist
-	   (member 'mew-refile-guess-by-thread mew-refile-guess-control))
-      (mew-lisp-save mew-refile-msgid-file-name mew-refile-msgid-alist)))
-
 (defun mew-refile-guess-subfolder-p (parent child)
   (string-match 
    (concat  "^" (regexp-quote (file-name-as-directory parent)))
@@ -412,6 +372,7 @@ as special. They use To: or Cc: instead of From:")
 ;;
 ;; search x in a nested list.
 ;;
+
 (defun mew-refile-guess-member-p (x lst)
   (catch 'found
     (while lst
@@ -420,13 +381,14 @@ as special. They use To: or Cc: instead of From:")
 	(if (mew-refile-guess-member-p x (car lst))
 	    (throw 'found t)))
        (t
-	(if (equal  x (car lst))
+	(if (equal x (car lst))
 	    (throw 'found t))))
       (setq lst (cdr lst)))))
 
 ;;
 ;; grep with regex in a list of string. (first match)
 ;;
+
 (defun mew-refile-match-in-list (regex lst)
   (let ((case-fold-search t))
     (catch 'found 
@@ -439,21 +401,26 @@ as special. They use To: or Cc: instead of From:")
 ;;;
 ;;; Summary mode
 ;;;
-(defun mew-refile-set (msg folder)
-  (let* ((msg-folders (assoc msg mew-summary-buffer-refile))
-	 (folders (cdr msg-folders)))
-    (if folders
-	(if (not (mew-folder-member folder folders))
-	    (setq mew-summary-buffer-refile 
-		  (cons (nreverse (cons folder (nreverse (cons msg folders))))
-			(delete msg-folders mew-summary-buffer-refile))))
-      (setq mew-summary-buffer-refile 
-	    (cons (list msg folder) mew-summary-buffer-refile)))))
 
-(defun mew-refile-reset (msg)
-  (setq mew-summary-buffer-refile
-	(delete (assoc msg mew-summary-buffer-refile)
-		mew-summary-buffer-refile)))
+(defun mew-refile-get (fld msg)
+  (save-excursion
+    (set-buffer fld)
+    (cdr (assoc msg (mew-sinfo-get-refile)))))
+
+(defun mew-refile-set (fld msg folders)
+  ;; mew-refile-reset MUST be called before this.
+  (when (get-buffer fld)
+    (save-excursion
+      (set-buffer fld)
+      (mew-sinfo-set-refile
+       (cons (cons msg folders) (mew-sinfo-get-refile))))))
+
+(defun mew-refile-reset (fld msg)
+  (when (get-buffer fld)
+    (save-excursion
+      (set-buffer fld)
+      (mew-sinfo-set-refile
+       (delq (assoc msg (mew-sinfo-get-refile)) (mew-sinfo-get-refile))))))
 
 ;; mew-refile-decide-folders returns: ((input1 input2...) 
 ;;                                     (guess-func-name1 guess1 guess2...)
@@ -462,27 +429,25 @@ as special. They use To: or Cc: instead of From:")
 ;;          'cdr' is a alist of opinions by guess functions.
 ;; cdr is needed for learning functions.
 ;;
-(defun mew-refile-decide-folders (buf msg mark &optional auto)
+
+(defun mew-refile-decide-folders (buf msg cur-folders &optional auto)
   (let (learn-info folders ret)
     (save-excursion
       (set-buffer buf)
       (setq learn-info (mew-refile-guess auto)))
-    (setq 
-     folders
-     (cond
+    (cond
+     (auto
       ;; if auto is set, simply use the guess.
-      (auto (car learn-info))
+      (setq folders (car learn-info)))
+     (cur-folders
       ;; add new folder 
-      ((equal mew-mark-refile mark)
-       (mew-input-folders 
-	nil (mew-join "," 
-		      (cdr (assoc msg mew-summary-buffer-refile)))))
+      (setq folders (mew-input-folders nil (mew-join "," cur-folders))))
+     ((nth 1 (car learn-info))
       ;; multi guess
-      ((nth 1 (car learn-info))
-       (mew-input-folders nil (mew-join "," (car learn-info))))
+      (setq folders (mew-input-folders nil (mew-join "," (car learn-info)))))
+     (t 
       ;; single guess
-      (t 
-       (mew-input-folders (nth 0 (car learn-info))))))
+      (setq folders (mew-input-folders (nth 0 (car learn-info))))))
     ;; check folder existence.
     (while folders
       (if (mew-folder-check (car folders))
@@ -490,8 +455,6 @@ as special. They use To: or Cc: instead of From:")
       (setq folders (cdr folders)))
     (cons (nreverse ret) (cdr learn-info)))) ;; return value
 
-(defvar mew-header-reasonable-size 5000)
-
 (defun mew-summary-refile (&optional report)
  "Put the refile mark(default is 'o') on this message. If already
 marked with 'o', it prints where this message will be refiled. This
@@ -499,111 +462,125 @@ can overlay other marks. When it overlays, the cursor stays on the
 message. If it marks newly, displays the next message. If executed
 with '\\[universal-argument]', it displays how the refile rules work in Message mode."
  (interactive "P")
- (if report (mew-summary-refile-report) (mew-summary-refile-body)))
+ (mew-summary-msg-or-part
+  (mew-summary-not-in-draft
+   (if report (mew-summary-refile-report) (mew-summary-refile-body)))))
 
-(defun mew-summary-refile-body (&optional exp-flds auto)
-  (mew-summary-only
-   (mew-summary-msg-or-part
-    (let (msg folders mark buf learn-info fld)
-      (save-excursion
-	;; save the cursor position anyway
-	(mew-summary-goto-message)
-	;; on the message
-	(setq fld (mew-summary-folder-name))
-	(setq msg (mew-summary-message-number)) ;; msg is never nil
-	(setq mark (mew-summary-get-mark))) ;; any mark
-      (if exp-flds
-	  (setq folders exp-flds)
-	;; show message if not displayed
-	(if (or auto (null mew-summary-buffer-disp-msg))
-	    (save-excursion
-	      (mew-set-buffer-tmp)
-	      (setq buf (current-buffer))
-	      (mew-insert-message fld msg mew-cs-autoconv
-				  mew-header-reasonable-size)
-	      (goto-char (point-min))
-	      (if (and (re-search-forward (concat "^$\\|^" mew-subj:) nil t)
-		       (not (looking-at "^$")))
-		  (let ((med (point)))
-		    (forward-line)
-		    (mew-header-goto-next)
-		    (mew-header-decode-region 'text med (point)))))
-	  (mew-summary-display nil) ;; cursor position is important
-	  (setq buf (or (mew-cache-hit (cons (buffer-name) msg))
-			(mew-buffer-message)))) ;; for ","
-	(setq learn-info (mew-refile-decide-folders buf msg mark auto))
-	(setq folders (car learn-info)))
-      ;; mark refile
-      (if folders
+(defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark)
+  (let (msg folders cur-folders mark buf learn-info fld tmp delbuf)
+    (save-excursion
+      ;; save the cursor position anyway
+      (mew-summary-goto-message)
+      ;; on the message
+      (setq fld (mew-summary-folder-name))
+      (setq msg (mew-summary-message-number));; msg is never nil
+      (setq mark (mew-summary-get-mark)));; any mark
+    (if exp-flds
+	(setq folders exp-flds)
+      ;; show message if not displayed
+      (if (or auto (null (mew-sinfo-get-disp-msg)))
 	  (save-excursion
-	    (mew-summary-goto-message)
-	    (or exp-flds auto (mew-refile-guess-learn buf learn-info))
-	    (mew-refile-reset msg)
-	    (mapcar (function (lambda (x) (mew-refile-set msg x))) folders)
-	    (mew-mark-unmark)
-	    (mew-summary-mark-as mew-mark-refile)))
-      ;; memorize last-folder
-      (setq mew-refile-last-folder folders)
-      (if (or mark auto (not folders))
-	  () ;; stay here
-	(mew-summary-goto-message)
-	;; on the message
-	(mew-decode-syntax-delete)
-	;; for C-x C-x
-	(beginning-of-line)
-	(let ((zmacs-regions nil))
-	  (push-mark (point) t t))
-	(mew-summary-display-after mew-summary-mark-direction))
-      (set-buffer-modified-p nil)
-      folders)))) ;; return value
+	    (setq buf (generate-new-buffer mew-buffer-prefix))
+	    (setq delbuf t)
+	    (set-buffer buf)
+	    (mew-erase-buffer)
+	    (mew-insert-message fld msg mew-cs-autoconv
+				mew-header-reasonable-size)
+	    (goto-char (point-min))
+	    (if (and (re-search-forward (concat "^$\\|^" mew-subj:) nil t)
+		     (not (looking-at "^$")))
+		(let ((med (point)))
+		  (forward-line)
+		  (mew-header-goto-next)
+		  (mew-header-decode-region mew-subj: med (point)))))
+	;; need to make a cache or a message buffer.
+	(mew-summary-display nil)
+	;; mew-cache-hit should be first since we want to get 
+	;; information form the top level header.
+	(setq buf (or (mew-cache-hit fld msg) (mew-buffer-message))))
+      (if (eq mew-mark-refile mark)
+	  (setq cur-folders (mew-refile-get fld msg)))
+      (setq learn-info (mew-refile-decide-folders buf msg cur-folders auto))
+      (setq folders (car learn-info)))
+    ;; we must prevent refiling a message to +queue
+    (while folders
+      (unless (mew-folder-queuep (car folders))
+	(setq tmp (cons (car folders) tmp)))
+      (setq folders (cdr folders)))
+    (setq folders (nreverse tmp))
+    (setq folders (delete mew-draft-folder folders))
+    ;; mark refile
+    (unless no-mark
+      (when folders
+	(save-excursion
+	  (mew-summary-goto-message)
+	  (or exp-flds auto (mew-refile-guess-learn buf learn-info))
+	  (mew-refile-reset fld msg)
+	  (mew-refile-set fld msg folders)
+	  (mew-mark-unmark))
+	(mew-mark-put-mark mew-mark-refile no-msg) ;; cursor may move
+	(mew-summary-mark-in-physical fld msg mew-mark-refile)))
+    (if delbuf (mew-remove-buffer buf))
+    ;; memorize last-folder
+    (setq mew-refile-last-folder folders)
+    (set-buffer-modified-p nil)
+    folders)) ;; return value
 
 (defun mew-summary-refile-report ()
-  (mew-summary-only
-   (mew-summary-msg-or-part
-    (let (fld msg guess buf
-	      (customize-var '(mew-refile-ctrl-multi
-			       mew-refile-guess-key-list
-			       mew-refile-guess-strip-domainpart
-			       mew-refile-guess-from-me-is-special)))
-      (save-excursion
-	;; save the cursor position anyway
-	(mew-summary-goto-message)
-	;; on the message
-	(setq fld (mew-summary-folder-name))
-	(setq msg (mew-summary-message-number)) ;; msg is never nil
-	(mew-set-buffer-tmp)
-	(mew-insert-message fld msg mew-cs-autoconv
-			    mew-header-reasonable-size)
-	(setq guess (mew-refile-guess nil t)))
-      (setq buf (buffer-name))
-      (mew-window-configure buf 'message)
-      (mew-elet
-       (mew-erase-buffer)
-       (save-excursion
-	 ;; report result of guess.
-	 (insert (format "** Guess result: %s\n" (car guess)))
-	 ;; report status of customize variables.
-	 (insert "\n** Current Configurations:\n\n")
-	 (while customize-var
-	   (insert (format "%-40s:  " (car customize-var)))
-	   (insert (format "%s\n"     (eval (car customize-var))))
-	   (setq customize-var (cdr customize-var)))
-	 (insert "\n** Each function's opinion:\n\n")
-	 ;; report how each functions guessed.
-	 (setq guess (cdr guess))
-	 (while guess
-	   (insert (format "%-32s  " (car (car guess))))
-	   (insert (format "return: %s\n" (cdr (car guess))))
-	   (setq guess (cdr guess)))))
-      (mew-message-clear-end-of)
-      (set-buffer-modified-p nil)
-      (mew-pop-to-buffer buf)))))
+  (let ((buf (current-buffer))
+	(tmpbuf (generate-new-buffer mew-buffer-prefix))
+	fld msg guess 
+	(customize-var '(mew-refile-ctrl-multi
+			 mew-refile-guess-key-list
+			 mew-refile-guess-strip-domainpart
+			 mew-refile-guess-from-me-is-special)))
+    (save-excursion
+      ;; save the cursor position anyway
+      (mew-summary-goto-message)
+      ;; on the message
+      (setq fld (mew-summary-folder-name))
+      (setq msg (mew-summary-message-number));; msg is never nil
+      (set-buffer tmpbuf)
+      (mew-erase-buffer)
+      (mew-insert-message fld msg mew-cs-autoconv
+			  mew-header-reasonable-size)
+      (setq guess (mew-refile-guess nil t)))
+    (mew-remove-buffer tmpbuf)
+    (mew-window-configure 'message)
+    ;; message buffer
+    (mew-elet
+     (mew-erase-buffer)
+     (save-excursion
+       ;; report result of guess.
+       (insert (format "** Guess result: %s\n" (car guess)))
+       ;; report status of customize variables.
+       (insert "\n** Current Configurations:\n\n")
+       (while customize-var
+	 (insert (format "%-40s:  " (car customize-var)))
+	 (insert (format "%s\n"     (eval (car customize-var))))
+	 (setq customize-var (cdr customize-var)))
+       (insert "\n** Each function's opinion:\n\n")
+       ;; report how each functions guessed.
+       (setq guess (cdr guess))
+       (while guess
+	 (insert (format "%-32s  " (car (car guess))))
+	 (insert (format "return: %s\n"
+			 (mapconcat (function identity) (cdr (car guess)) ",")))
+	 (setq guess (cdr guess)))))
+    (mew-message-clear-end-of)
+    (set-buffer-modified-p nil)
+    (mew-pop-to-buffer buf)))
 
 (defun mew-summary-refile-again ()
   "Put a refile mark on this message according to the previous 
 refile folder."
   (interactive)
-  (mew-summary-only
+  (mew-summary-not-in-draft
+   (save-excursion
+     (mew-summary-goto-message)
+     (message "%s to %s"
+	      (mew-summary-message-number)
+	      (mew-join ", " mew-refile-last-folder)))
    (mew-summary-refile-body mew-refile-last-folder)))
 
 (defun mew-summary-auto-refile (&optional mew-mark-review-only)
@@ -613,70 +590,69 @@ any previousely marked message will be skipped.
 If '\\[universal-argument]' is specified, only messages marked with
 'mew-mark-review' will be conserned."
   (interactive "P")
-  (mew-summary-only
-   (let ((after-change-function nil)
-	 (after-change-functions nil)
-	 (mew-use-highlight-x-face nil)
-	 (lines (count-lines (point-min) (point-max)))
-	 (line 1) (mark nil) msg)
-     (cond
-      (mew-mark-review-only
-       (setq msg (format "Refile all messages marked with '%c'? "
-			 mew-mark-review)))
-      (mew-refile-auto-refile-skip-any-mark
-       (setq msg (format "Refile all non-marked messages? ")))
-      (t
-       (setq msg (format "Refile all messages except those marked with '%c' and '%c'? " mew-mark-refile mew-mark-delete))))
-     (if (and mew-refile-auto-refile-confirm (null (yes-or-no-p msg)))
-	 (message "Not refiled.")
-       (message "Auto refiling ...")
-       (save-window-excursion
-	 (goto-char (point-min))
-	 (while (not (eobp))
-	   (setq mark (mew-summary-get-mark))
-	   (if mew-mark-review-only 
-	       (and (equal mark mew-mark-review)
-		    (mew-summary-refile-body nil t))
-	     (or (equal mark mew-mark-refile)
-		 (equal mark mew-mark-delete)
-		 (and mark mew-refile-auto-refile-skip-any-mark)
-		 (mew-summary-refile-body nil t)))
-	   (forward-line)
-	   (if (equal (% (/ (* 100 line) lines) 10) 0)
-	       (message "Auto refiling ... %s%%"
-			(/ (* 100 line) lines)))
-	   (setq line (1+ line)))
-	 (message "Auto refiling ... done"))))))
+  (mew-summary-or-thread
+   (mew-summary-not-in-draft
+    (mew-decode-syntax-delete)
+    (let ((after-change-functions nil)
+	  (mew-use-highlight-x-face nil)
+	  (lines (mew-count-lines (point-min) (point-max)))
+	  (line 1) (mark nil) msg)
+      (cond
+       (mew-mark-review-only
+	(setq msg (format "Refile all messages marked with '%c'? "
+			  mew-mark-review)))
+       (mew-refile-auto-refile-skip-any-mark
+	(setq msg (format "Refile all non-marked messages? ")))
+       (t
+	(setq msg (format "Refile all messages except those marked with '%c' and '%c'? " mew-mark-refile mew-mark-delete))))
+      (if (and mew-refile-auto-refile-confirm (null (yes-or-no-p msg)))
+	  (message "Not refiled.")
+	(message "Auto refiling ... ")
+	(save-window-excursion
+	  (goto-char (point-min))
+	  (while (not (eobp))
+	    (when (looking-at mew-regex-msg)
+	      (setq mark (mew-summary-get-mark))
+	      (if mew-mark-review-only 
+		  (and (eq mark mew-mark-review);; xxx char-equal?
+		       (mew-summary-refile-body nil t 'no-msg))
+		(or (eq mark mew-mark-refile)
+		    (eq mark mew-mark-delete)
+		    (and mark mew-refile-auto-refile-skip-any-mark)
+		    (mew-summary-refile-body nil t 'no-msg)))
+	      (if (= (% (/ (* 100 line) lines) 10) 0)
+		  (message "Auto refiling ... %s%%"
+			   (/ (* 100 line) lines)))
+	      (setq line (1+ line)))
+	    (forward-line))
+	  (message "Auto refiling ... done")))))))
 
 ;;
 ;; "mx" extension
 ;;
+
 (defun mew-summary-mark-refile ()
   "\\<mew-summary-mode-map>
 Put the refile mark onto all messages marked with '*'.
 This is very convenient to refile all messages picked by '\\[mew-summary-search-mark]'."
   (interactive)
-  (mew-summary-only
-   (let ((after-change-function nil)
-	 (after-change-functions nil)
-	 (mew-use-highlight-x-face nil)
-	 (last nil)
-	 (regex (concat mew-summary-message-regex
-			(regexp-quote (char-to-string mew-mark-review)))))
-     (message "Mark refiling ...")
-     (save-excursion
-       (save-window-excursion
-	 (goto-char (point-min))
-	 (while (re-search-forward regex nil t)
-	   (setq last (mew-summary-refile-body last))
-	   (forward-line))
-	 (message "Mark refiling ... done"))))))
+  (let ((after-change-functions nil)
+	(mew-use-highlight-x-face nil)
+	(last nil))
+    (message "Mark refiling ... ")
+    (save-excursion
+      (save-window-excursion
+	(goto-char (point-min))
+	(while (re-search-forward mew-regex-msg-review nil t)
+	  (setq last (mew-summary-refile-body last nil 'no-msg))
+	  (forward-line))
+	(message "Mark refiling ... done")))))
 
 (provide 'mew-refile)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1994-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
