@@ -2,12 +2,10 @@
 
 ;; Author:  Shuichi Kitaguchi <kit@Mew.org>
 ;; Created: Dec 05, 1997
-;; Revised: Nov 20, 1999
+;; Revised: Jul  8, 2001
 
 ;;; Code:
 
-(defconst mew-win32-version "mew-win32.el 0.33.1")
-
 ;;; for NTEmacs User
 ;;
 ;; put mw32script.el(in Meadow's archive) into load-path.
@@ -17,17 +15,6 @@
 ;;  ~/.emacs settings
 ;;
 
-;;; for PGP/MIME
-;;    mew-prog-pgp           ...  executable name for PGP/MIME
-;;
-;;; example
-;;    (setq mew-prog-pgp  "pgp.exe")      ;; PGP
-;;    (setq mew-prog-pgp  "gpg.exe")      ;; GNUPG
-
-;;; for Windows NT with NTFS
-;;(eval-after-load "mew-win32"
-;;  '(setq mew-touch-folder-p nil))
-
 ;;; for PRINTING
 ;;    mew-w32-prog-print     ...  print command
 ;;    mew-w32-prog-print-arg ...  print command argument
@@ -52,28 +39,26 @@
 
 ;; Emacs version dependent variables.
 (cond
-;; ((featurep 'xemacs)	; XEmacs?
-;;  (setq mew-prog-shell-arg  shell-command-switch)
+ ((featurep 'xemacs)			; XEmacs
+  (setq mew-prog-shell-arg  shell-command-switch))
   
- ((eq 20 emacs-major-version)
-  (if (boundp 'w32-system-coding-system)
-      (setq mew-cs-pick w32-system-coding-system))
+ ((<= 20 emacs-major-version)
   (setq mew-prog-shell-arg  shell-command-switch)
   
-  (if (featurep 'meadow);; Meadow
+  (if (featurep 'meadow) ;; Meadow
       (progn
 	(require 'mw32script)
 	(mw32script-init))
-    (if (require 'mw32script nil t);; NTEmacs
+    (if (require 'mw32script nil t) ;; NTEmacs
 	(progn
 	  (mw32script-make-pathext-regexp)
-	  (fset 'call-process-original (symbol-function 'call-process))
+	  (defalias 'call-process-original 'call-process)
 	  (defun call-process (PROGRAM INFILE BUFFER DISPLAY &rest PROGRAM-ARGS)
 	    (let (prog sargs)
 	      (setq prog (mw32script-openp PROGRAM))
 	      (if (not prog)
 		  (progn
-		    (setq prog (mew-which PROGRAM exec-path))
+		    (setq prog (mew-which-exec PROGRAM))
 		    (setq sargs (mw32script-resolve-script prog))))
 	      (if sargs
 		  (apply (function call-process-original)
@@ -81,57 +66,26 @@
 			 prog PROGRAM-ARGS)
 		(apply (function call-process-original)
 		       PROGRAM INFILE BUFFER DISPLAY PROGRAM-ARGS))))
-	  (fset 'start-process-original (symbol-function 'start-process))
+	  (defalias 'start-process-original 'start-process)
 	  (defun start-process (NAME BUFFER PROGRAM &rest PROGRAM-ARGS)
 	    (let (prog sargs)
 	      (setq prog (mw32script-openp PROGRAM))
 	      (if (not prog)
 		  (progn
-		    (setq prog (mew-which PROGRAM exec-path))
+		    (setq prog (mew-which-exec PROGRAM))
 		    (setq sargs (mw32script-resolve-script prog))))
 	      (if sargs
 		  (apply (function start-process-original)
 			 NAME BUFFER (car sargs) prog PROGRAM-ARGS)
 		(apply (function start-process-original)
 		       NAME BUFFER PROGRAM PROGRAM-ARGS))))))
-    ))
- (t				 ;; Mule for Windows
-  (setq mew-cs-pick         win32-system-coding-system)
-  (setq mew-use-timer       nil)
-  (setq mew-prog-shell-arg  shell-command-option)
-  (require 'win32-script)
-  (define-process-argument-editing
-    "/[^./]+$"
-    'script-process-argument-editing-function 'last)))
+    )))
 
 ;; common programs.
-(setq mew-prog-mime-encode  "mewencode.exe")
-(setq mew-prog-mime-decode  "mewdecode.exe")
-(setq mew-prog-tar          "tar.exe")
-(setq mew-prog-compress     "compress.exe")
-(setq mew-prog-gzip         "gzip.exe")
 (setq mew-prog-shell        shell-file-name)
 
-;(setq mew-prog-pgp          "pgp.exe")
-(setq mew-prog-pgp2         "pgp.exe")
-(setq mew-prog-pgp5         "pgp.exe")
-(setq mew-prog-pgp5e        "pgpe.exe")
-(setq mew-prog-pgp5s        "pgps.exe")
-(setq mew-prog-pgp5v        "pgpv.exe")
-(setq mew-prog-pgp5k        "pgpk.exe")
-(setq mew-prog-gpg          "gpg.exe")
-
-(setq mew-touch-folder-p    t)
 (setq mew-delete-temp-file  nil)
 
-(setq mew-prog-uncompface   "uncompface.exe")
-
-(setq mew-prog-text/html     mew-w32-exec)
-(setq mew-prog-text/html-arg nil)
-(setq mew-ext-prog-url       mew-w32-exec)
-(setq mew-ext-prog-url-args  nil)
-
-
 ;; printing
 (defun mew-w32-print-buffer ()
   (interactive)
@@ -150,31 +104,40 @@
 
 
 ;; MIME setting
-(defvar mew-prog-plain           '(mew-mime-text/plain     () nil))
-(defvar mew-prog-html            '(mew-mime-text/html      () nil))
-(defvar mew-prog-enriched        '(mew-mime-text/enriched  () nil))
-(defvar mew-prog-text            '(mew-mime-text/plain     () nil))
-(defvar mew-prog-audio           (list mew-w32-exec        () t))
-(defvar mew-prog-audio2          (list mew-w32-exec        () t))
-(defvar mew-prog-gif             (list mew-w32-exec        () t))
-(defvar mew-prog-tiff            (list mew-w32-exec        () t))
-(defvar mew-prog-jpeg            (list mew-w32-exec        () t))
-(defvar mew-prog-xwd             (list mew-w32-exec        () t))
-(defvar mew-prog-xbm             (list mew-w32-exec        () t))
-(defvar mew-prog-xpm             (list mew-w32-exec        () t))
-(defvar mew-prog-png             (list mew-w32-exec        () t))
-(defvar mew-prog-bmp             (list mew-w32-exec        () t))
-(defvar mew-prog-image           (list mew-w32-exec        () t))
-(defvar mew-prog-mpeg            (list mew-w32-exec        () t))
-(defvar mew-prog-rfc822          '(mew-mime-message/rfc822 () nil))
-(defvar mew-prog-external-body   '(mew-mime-external-body  () nil))
-(defvar mew-prog-delivery-status '(mew-mime-text/plain     () nil))
-(defvar mew-prog-postscript      (list mew-w32-exec        () t))
-(defvar mew-prog-pgp-keys        '(mew-mime-pgp-keys       () nil))
-(defvar mew-prog-pdf             (list mew-w32-exec        () t))
-(defvar mew-prog-octet-stream    (list mew-w32-exec        () t))
+(defvar mew-prog-plain    '(mew-mime-text/plain mew-mime-text/plain-ext))
+(defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
+(defvar mew-prog-enriched 'mew-mime-text/enriched)
+(defvar mew-prog-text     'mew-mime-text/plain)
+(defvar mew-prog-image           '(mew-mime-image/* mew-mime-image/*-ext))
+(defvar mew-prog-mpeg            `(,mew-w32-exec           () t))
+(defvar mew-prog-rfc822          'mew-mime-message/rfc822)
+(defvar mew-prog-external-body   '(mew-mime-external-body mew-mime-external-body-ext))
+(defvar mew-prog-delivery-status 'mew-mime-text/plain)
+(defvar mew-prog-postscript      `(,mew-w32-exec           () t))
+(defvar mew-prog-pgp-keys        '(mew-mime-pgp-keys mew-mime-pgp-keys-ext))
+(defvar mew-prog-pdf             `(,mew-w32-exec           () t))
+(defvar mew-prog-octet-stream    `(,mew-w32-exec           () t))
+(defvar mew-prog-msword          `(,mew-w32-exec           () t))
+(defvar mew-prog-msexcel         `(,mew-w32-exec           () t))
+(defvar mew-prog-mspowerpoint    `(,mew-w32-exec           () t))
 
-(defvar mew-prog-w32             (list mew-w32-exec        () t))
+(defvar mew-prog-w32             `(,mew-w32-exec           () t))
+
+;;;
+;;; Text/Html, Image
+;;;
+
+(defvar mew-prog-text/html         'mew-mime-text/html-w3m) ;; See w3m.el
+(defvar mew-prog-text/html-ext     mew-w32-exec)
+(defvar mew-prog-text/html-ext-arg nil)
+
+(defvar mew-prog-image/*         'mew-mime-image/*)
+(defvar mew-prog-image/*-ext     mew-w32-exec)
+(defvar mew-prog-image/*-ext-arg nil)
+
+(defvar mew-prog-application/msword nil)
+(defvar mew-prog-application/msexcel nil)
+(defvar mew-prog-application/mspowerpoint nil)
 
 (setq mew-mime-content-type
       (append
@@ -188,31 +151,29 @@
 	 ("image/x-tga"     "\\.tga$"   mew-b64 mew-prog-image mew-icon-image)
 	 ;; Video (mew-prog-mpeg)
 	 ("video/x-msvideo" "\\.avi$"   mew-b64 mew-prog-mpeg  mew-icon-video)
-	 ("video/quicktime" "\\.mov$"   mew-b64 mew-prog-mpeg  mew-icon-video)
 	 ;; Executable (mew-prog-w32)
-	 ("application/octet-stream" "\\.exe$\\|\\.com$\\|\\.cmd$\\|\\.bat$"
+	 ("application/octet-stream" "\\.exe$\\|\\.com$\\|\\.cmd$\\|\\.bat$\\|\\.dll$"
 	  mew-b64 mew-prog-w32 mew-icon-application/octet-stream)
 	 ;; Archive (mew-prog-w32)
-	 ("application/octet-stream"
-	  "\\.taz$|\\.tar\\.?b?z?2?$\\|\\.bz2?$\\|\\.arj$\\|\\.zoo$\\|\\.rar$|\\.cab$"
+	 ("application/octet-stream" "\\.arj$\\|\\.zoo$\\|\\.rar$|\\.cab$"
 	  mew-b64 mew-prog-w32 mew-icon-application/octet-stream)
 	 ;; Document (mew-prog-w32)
 	 ("application/x-dvi" "\\.dvi$"
 	  mew-b64 mew-prog-w32 mew-icon-application/postscript)
-	 ;; MS Application (mew-prog-w32)
-	 ("application/msword" "\\.doc$"
-	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ("application/vnd.ms-excel" "\\.xls$"
-	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ("application/vnd.ms-powerpoint" "\\.ppt$"
-	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ;; OASYS
+	 ;; OASYS (mew-prog-w32)
 	 ("application/vnd.fujitsu.oasys" "\\.oas$"
 	  mew-b64 mew-prog-w32 mew-icon-text)
 	 ("application/vnd.fujitsu.oasys2" "\\.oa2$"
 	  mew-b64 mew-prog-w32 mew-icon-text)
-	 ("application/vnd.fujitsu.oasys3" "\\.oa3$"
+	 ;; Ichitaro V.2,3,4,5,6,7,8,9,10,Lite,Lite2 (mew-prog-w32)
+	 ("application/octet-stream" "\\.jtd$\\|\\.jfw$\\|\\.jbw$\\|\\.jaw$\\|\\.jsw$\\|\\.jxw$\\|\\.jtt$\\|\\.jvw$\\|\\.juw$\\|\\.jtw$"
 	  mew-b64 mew-prog-w32 mew-icon-text)
+	 ;; Shockwave (mew-prog-w32)
+	 ("application/x-shockwave-flash" "\\.swf$"
+	  mew-b64 mew-prog-w32 mew-icon-image)
+	 ;; MPEG audio (mew-prog-w32)
+	 ("audio/x-mpeg" "\\.mpga$\\|\\.mp[23]$"
+	  mew-b64 mew-prog-w32 mew-icon-image)
 	 )
        mew-mime-content-type))
 
@@ -224,24 +185,36 @@
 	 ;; Audio
 	 "audio/x-wav" "audio/x-aiff" "audio/x-midi"
 	 ;; Video
-	 "video/x-msvideo" "video/quicktime"
+	 "video/x-msvideo"
 	 ;; Document
-	 "application/pdf" "application/x-dvi"
-	 ;; MS Application
-	 "application/msword" "application/vnd.ms-excel"
-	 "application/vnd.ms-powerpoint"
+	 "application/x-dvi"
 	 ;; OASYS
 	 "application/vnd.fujitsu.oasys"
 	 "application/vnd.fujitsu.oasys2"
-	 "application/vnd.fujitsu.oasys3"
+	 ;; Shockwave
+	 "application/x-shockwave-flash"
+	 ;; MPEG audio
+	 "audio/x-mpeg" "audio/mpeg"
 	 )
        mew-mime-content-type-list))
 
+(cond
+ ((eq mew-mule-ver 2)
+  (setq mew-cs-database-for-arg
+	'((*iso-2022-jp* . *sjis*unix)
+	  (*iso-2022-kr* . *euc-korea*unix))))
+ ((eq mew-mule-ver 3)
+  (setq mew-cs-database-for-arg
+	'((iso-2022-jp . shift_jis-unix)
+	  (iso-2022-kr . euc-kr-unix)))))
+
+(setq mew-prog-grep-max-msgs 2000) ;; for external grep (pick & virtual)
+
 (provide 'mew-win32)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996, 1997, 1998, 1999 Mew developing team.
+;; Copyright (C) 1996-2001 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
